@@ -276,9 +276,8 @@ swfdec_gtk_widget_expose (GtkWidget *gtkwidget, GdkEventExpose *event)
     cairo_surface_set_device_offset (surface, -event->area.x, -event->area.y);
     cr = cairo_create (surface);
   }
-  gdk_cairo_region (cr, event->region);
-  cairo_clip (cr);
-  swfdec_player_render (priv->player, cr);
+  swfdec_player_render (priv->player, cr,
+      event->area.x, event->area.y, event->area.width, event->area.height);
   cairo_show_page (cr);
   cairo_destroy (cr);
 
@@ -684,7 +683,7 @@ swfdec_gtk_widget_do_invalidate (gpointer widgetp)
 }
 
 static void
-swfdec_gtk_widget_invalidate_cb (SwfdecPlayer *player,
+swfdec_gtk_widget_invalidate_cb (SwfdecPlayer *player, const SwfdecRectangle *extents,
     const SwfdecRectangle *rect, guint n_rects, SwfdecGtkWidget *widget)
 {
   SwfdecGtkWidgetPrivate *priv = widget->priv;
