@@ -162,9 +162,6 @@ jpeg_decoder_init_decoder (JpegDecoder *dec)
     return;
   }
 
-
-
-
   for (i=0; i < dec->n_components; i++) {
     max_h_sample = MAX (max_h_sample, dec->components[i].h_sample);
     max_v_sample = MAX (max_v_sample, dec->components[i].v_sample);
@@ -995,12 +992,12 @@ jpeg_decoder_addbits (JpegDecoder * dec, unsigned char *data, unsigned int len)
 }
 
 int
-jpeg_decoder_get_image_size (JpegDecoder * dec, int *width, int *height)
+jpeg_decoder_get_image_size (JpegDecoder * dec, unsigned int *width, unsigned int *height)
 {
   if (width)
-    *width = dec->width;
+    *width = (unsigned) dec->width;
   if (height)
-    *height = dec->height;
+    *height = (unsigned) dec->height;
 
   return 0;
 }
