@@ -1,5 +1,5 @@
 /* Swfdec
- * Copyright (C) 2007-2008 Benjamin Otte <otte@gnome.org>
+ * Copyright (C) 2007 Benjamin Otte <otte@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -121,10 +121,11 @@ gboolean	swfdec_as_context_catch		(SwfdecAsContext *	context,
 
 gboolean	swfdec_as_context_try_use_mem	(SwfdecAsContext *	context,
 						 gsize			bytes);
-void		swfdec_as_context_use_mem	(SwfdecAsContext *	context,
+gboolean	swfdec_as_context_use_mem	(SwfdecAsContext *	context,
 						 gsize			bytes);
 void		swfdec_as_context_unuse_mem   	(SwfdecAsContext *	context,
 						 gsize			bytes);
+void		swfdec_as_object_mark		(SwfdecAsObject *	object);
 void		swfdec_as_value_mark		(SwfdecAsValue *	value);
 void		swfdec_as_string_mark		(const char *		string);
 void		swfdec_as_context_gc		(SwfdecAsContext *	context);
@@ -132,6 +133,14 @@ void		swfdec_as_context_maybe_gc	(SwfdecAsContext *	context);
 
 void		swfdec_as_context_run		(SwfdecAsContext *	context);
 
+void		swfdec_as_context_eval		(SwfdecAsContext *	context,
+						 SwfdecAsObject *	obj,
+						 const char *		str,
+						 SwfdecAsValue *	val);
+void		swfdec_as_context_eval_set	(SwfdecAsContext *	context,
+						 SwfdecAsObject *	obj,
+						 const char *		str,
+						 const SwfdecAsValue *	val);
 
 G_END_DECLS
 #endif
