@@ -29,7 +29,6 @@
 #include "swfdec_as_strings.h"
 #include "swfdec_as_super.h"
 #include "swfdec_debug.h"
-#include "swfdec_movie.h"
 
 /**
  * SECTION:SwfdecAsFrame
@@ -87,7 +86,7 @@ swfdec_as_stack_iterator_init_arguments (SwfdecAsStackIterator *iter, SwfdecAsFr
     iter->current = NULL;
     return NULL;
   }
-  context = swfdec_gc_object_get_context (frame);
+  context = SWFDEC_AS_OBJECT (frame)->context;
   if (frame->argv) {
     iter->stack = NULL;
     iter->current = (SwfdecAsValue *) frame->argv;
@@ -134,7 +133,7 @@ swfdec_as_stack_iterator_init (SwfdecAsStackIterator *iter, SwfdecAsFrame *frame
   g_return_val_if_fail (iter != NULL, NULL);
   g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), NULL);
 
-  context = swfdec_gc_object_get_context (frame);
+  context = SWFDEC_AS_OBJECT (frame)->context;
   iter->i = 0;
   stack = context->stack;
   if (context->frame == frame) {
@@ -293,27 +292,27 @@ swfdec_as_frame_dispose (GObject *object)
 }
 
 static void
-swfdec_as_frame_mark (SwfdecGcObject *object)
+swfdec_as_frame_mark (SwfdecAsObject *object)
 {
   SwfdecAsFrame *frame = SWFDEC_AS_FRAME (object);
   guint i;
 
   if (frame->next)
-    swfdec_gc_object_mark (frame->next);
-  g_slist_foreach (frame->scope_chain, (GFunc) swfdec_gc_object_mark, NULL);
+    swfdec_as_object_mark (SWFDEC_AS_OBJECT (frame->next));
+  g_slist_foreach (frame->scope_chain, (GFunc) swfdec_as_object_mark, NULL);
   if (frame->thisp)
-    swfdec_gc_object_mark (frame->thisp);
+    swfdec_as_object_mark (frame->thisp);
   if (frame->super)
-    swfdec_gc_object_mark (frame->super);
-  swfdec_gc_object_mark (frame->target);
-  swfdec_gc_object_mark (frame->original_target);
+    swfdec_as_object_mark (frame->super);
+  swfdec_as_object_mark (frame->target);
+  swfdec_as_object_mark (frame->original_target);
   if (frame->function)
-    swfdec_gc_object_mark (frame->function);
+    swfdec_as_object_mark (SWFDEC_AS_OBJECT (frame->function));
   for (i = 0; i < frame->n_registers; i++) {
     swfdec_as_value_mark (&frame->registers[i]);
   }
   /* don't mark argv, it's const, others have to take care of it */
-  SWFDEC_GC_OBJECT_CLASS (swfdec_as_frame_parent_class)->mark (object);
+  SWFDEC_AS_OBJECT_CLASS (swfdec_as_frame_parent_class)->mark (object);
 }
 
 static char *
@@ -356,13 +355,11 @@ static void
 swfdec_as_frame_class_init (SwfdecAsFrameClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  SwfdecGcObjectClass *gc_class = SWFDEC_GC_OBJECT_CLASS (klass);
   SwfdecAsObjectClass *asobject_class = SWFDEC_AS_OBJECT_CLASS (klass);
 
   object_class->dispose = swfdec_as_frame_dispose;
 
-  gc_class->mark = swfdec_as_frame_mark;
-
+  asobject_class->mark = swfdec_as_frame_mark;
   asobject_class->debug = swfdec_as_frame_debug;
 }
 
@@ -377,7 +374,7 @@ swfdec_as_frame_init (SwfdecAsFrame *frame)
 static void
 swfdec_as_frame_load (SwfdecAsFrame *frame)
 {
-  SwfdecAsContext *context = swfdec_gc_object_get_context (frame);
+  SwfdecAsContext *context = SWFDEC_AS_OBJECT (frame)->context;
 
   frame->stack_begin = context->cur;
   context->base = frame->stack_begin;
@@ -396,7 +393,10 @@ swfdec_as_frame_new (SwfdecAsContext *context, SwfdecScript *script)
   g_return_val_if_fail (script != NULL, NULL);
   
   size = sizeof (SwfdecAsFrame) + sizeof (SwfdecAsValue) * script->n_registers;
-  frame = g_object_new (SWFDEC_TYPE_AS_FRAME, "context", context, NULL);
+  if (!swfdec_as_context_use_mem (context, size))
+    return NULL;
+  frame = g_object_new (SWFDEC_TYPE_AS_FRAME, NULL);
+  swfdec_as_object_add (SWFDEC_AS_OBJECT (frame), context, size);
   frame->script = swfdec_script_ref (script);
   frame->function_name = script->name;
   SWFDEC_DEBUG ("new frame for function %s", frame->function_name);
@@ -424,8 +424,11 @@ swfdec_as_frame_new_native (SwfdecAsContext *context)
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), NULL);
   
   size = sizeof (SwfdecAsFrame);
-  frame = g_object_new (SWFDEC_TYPE_AS_FRAME, "context", context, NULL);
+  if (!swfdec_as_context_use_mem (context, size))
+    return NULL;
+  frame = g_object_new (SWFDEC_TYPE_AS_FRAME, NULL);
   SWFDEC_DEBUG ("new native frame");
+  swfdec_as_object_add (SWFDEC_AS_OBJECT (frame), context, size);
   swfdec_as_frame_load (frame);
   return frame;
 }
@@ -448,7 +451,7 @@ swfdec_as_frame_return (SwfdecAsFrame *frame, SwfdecAsValue *return_value)
   SwfdecAsFrame *next;
 
   g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
-  context = swfdec_gc_object_get_context (frame);
+  context = SWFDEC_AS_OBJECT (frame)->context;
   g_return_if_fail (frame == context->frame);
 
   /* save return value in case it was on the stack somewhere */
@@ -554,35 +557,24 @@ swfdec_as_frame_get_variable_and_flags (SwfdecAsFrame *frame, const char *variab
     SwfdecAsValue *value, guint *flags, SwfdecAsObject **pobject)
 {
   GSList *walk;
-  SwfdecAsContext *cx;
 
   g_return_val_if_fail (SWFDEC_IS_AS_FRAME (frame), NULL);
   g_return_val_if_fail (variable != NULL, NULL);
 
-  cx = swfdec_gc_object_get_context (frame);
-
   for (walk = frame->scope_chain; walk; walk = walk->next) {
     if (swfdec_as_object_get_variable_and_flags (walk->data, variable, value, 
 	  flags, pobject))
       return walk->data;
   }
   /* we've walked the scope chain down. Now look in the special objects. */
-  /* 1) the target (if removed, use original target) */
-  if (SWFDEC_IS_MOVIE (frame->target) &&
-      SWFDEC_MOVIE(frame->target)->state < SWFDEC_MOVIE_STATE_DESTROYED) {
-    if (swfdec_as_object_get_variable_and_flags (frame->target, variable,
-	  value, flags, pobject))
-      return frame->target;
-  } else {
-    if (swfdec_as_object_get_variable_and_flags (frame->original_target,
-	  variable, value, flags, pobject))
-      return frame->original_target;
-  }
+  /* 1) the target */
+  if (swfdec_as_object_get_variable_and_flags (frame->target, variable, value, 
+	flags, pobject))
+    return frame->target;
   /* 2) the global object */
-  /* FIXME: ignored on version 4, but should it never be created instead? */
-  if (cx->version > 4 && swfdec_as_object_get_variable_and_flags (cx->global,
-	variable, value, flags, pobject))
-    return swfdec_gc_object_get_context (frame)->global;
+  if (swfdec_as_object_get_variable_and_flags (
+	SWFDEC_AS_OBJECT (frame)->context->global, variable, value, flags, pobject))
+    return SWFDEC_AS_OBJECT (frame)->context->global;
 
   return NULL;
 }
@@ -643,7 +635,7 @@ swfdec_as_frame_delete_variable (SwfdecAsFrame *frame, const char *variable)
   if (ret)
     return ret;
   /* 2) the global object */
-  return swfdec_as_object_delete_variable (swfdec_gc_object_get_context (frame)->global, variable);
+  return swfdec_as_object_delete_variable (SWFDEC_AS_OBJECT (frame)->context->global, variable);
 }
 
 /**
@@ -684,7 +676,7 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
 
   /* setup */
   object = SWFDEC_AS_OBJECT (frame);
-  context = swfdec_gc_object_get_context (frame);
+  context = object->context;
   script = frame->script;
   if (frame->script == NULL)
     goto out;
@@ -693,6 +685,8 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
   if ((script->flags & (SWFDEC_SCRIPT_PRELOAD_ARGS | SWFDEC_SCRIPT_SUPPRESS_ARGS)) != SWFDEC_SCRIPT_SUPPRESS_ARGS) {
     SwfdecAsFrame *next;
     args = swfdec_as_array_new (context);
+    if (!args)
+      goto out;
     for (cur = swfdec_as_stack_iterator_init_arguments (&iter, frame); cur != NULL;
 	cur = swfdec_as_stack_iterator_next (&iter)) {
       swfdec_as_array_push (SWFDEC_AS_ARRAY (args), cur);
@@ -708,16 +702,14 @@ swfdec_as_frame_preload (SwfdecAsFrame *frame)
     } else {
       SWFDEC_AS_VALUE_SET_NULL (&val);
     }
-    swfdec_as_object_set_variable_and_flags (args, SWFDEC_AS_STR_caller, &val,
-	SWFDEC_AS_VARIABLE_HIDDEN | SWFDEC_AS_VARIABLE_PERMANENT);
+    swfdec_as_object_set_variable (args, SWFDEC_AS_STR_caller, &val);
 
     if (frame->function != NULL) {
       SWFDEC_AS_VALUE_SET_OBJECT (&val, SWFDEC_AS_OBJECT (frame->function));
     } else {
       SWFDEC_AS_VALUE_SET_NULL (&val);
     }
-    swfdec_as_object_set_variable_and_flags (args, SWFDEC_AS_STR_callee, &val,
-	SWFDEC_AS_VARIABLE_HIDDEN | SWFDEC_AS_VARIABLE_PERMANENT);
+    swfdec_as_object_set_variable (args, SWFDEC_AS_STR_callee, &val);
   } else {
     /* silence gcc */
     args = NULL;
@@ -824,7 +816,7 @@ swfdec_as_frame_handle_exception (SwfdecAsFrame *frame)
   SwfdecAsContext *cx;
 
   g_return_if_fail (SWFDEC_IS_AS_FRAME (frame));
-  cx = swfdec_gc_object_get_context (frame);
+  cx = SWFDEC_AS_OBJECT (frame)->context;
   g_return_if_fail (cx->exception);
 
   /* pop blocks in the hope that we are inside a Try block */
