@@ -1,5 +1,5 @@
 /* Swfdec
- * Copyright (C) 2007-2008 Benjamin Otte <otte@gnome.org>
+ * Copyright (C) 2007 Benjamin Otte <otte@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,7 +22,6 @@
 
 #include <glib-object.h>
 #include <swfdec/swfdec_as_types.h>
-#include <swfdec/swfdec_gc_object.h>
 
 G_BEGIN_DECLS
 
@@ -58,18 +57,25 @@ typedef gboolean (* SwfdecAsVariableForeach) (SwfdecAsObject *object,
 
 struct _SwfdecAsObject {
   /*< protected >*/
-  SwfdecGcObject      	object;
+  GObject		object;
+  SwfdecAsContext *	context;	/* context the object belongs to */
   /*< private >*/
   SwfdecAsObject *	prototype;	/* prototype object (referred to as __proto__) */
   guint			prototype_flags; /* propflags for the prototype object */
   GHashTable *		properties;	/* string->SwfdecAsVariable mapping or NULL when not in GC */
   GHashTable *		watches;	/* string->WatchData mapping or NULL when not watching anything */
+  guint8		flags;		/* GC flags */
+  gsize			size;		/* size reserved in GC */
   GSList *		interfaces;	/* list of interfaces this object implements */
 };
 
 struct _SwfdecAsObjectClass {
-  SwfdecGcObjectClass	object_class;
+  GObjectClass		object_class;
 
+  /* mark everything that should survive during GC */
+  void			(* mark)		(SwfdecAsObject *	object);
+  /* object was added to the context */
+  void			(* add)			(SwfdecAsObject *	object);
   /* get the value and flags for a variables */
   gboolean	      	(* get)			(SwfdecAsObject *       object,
 						 SwfdecAsObject *	orig,
@@ -112,6 +118,10 @@ void		swfdec_as_object_set_constructor(SwfdecAsObject *	object,
 SwfdecAsObject *swfdec_as_object_resolve	(SwfdecAsObject *	object);
 char *		swfdec_as_object_get_debug	(SwfdecAsObject *	object);
 
+void		swfdec_as_object_add		(SwfdecAsObject *     	object,
+						 SwfdecAsContext *    	context,
+						 gsize			size);
+
 /* I'd like to name these [gs]et_property, but binding authors will complain
  * about overlap with g_object_[gs]et_property then */
 #define swfdec_as_object_set_variable(object, variable, value) \
@@ -167,7 +177,7 @@ SwfdecAsFunction *swfdec_as_object_add_constructor
 						 guint			min_args,
 						 SwfdecAsObject *	prototype);
 
-gboolean	swfdec_as_object_call		(SwfdecAsObject *       object,
+void		swfdec_as_object_call		(SwfdecAsObject *       object,
 						 const char *		name,
 						 guint			argc,
 						 SwfdecAsValue *	argv,
