@@ -35,13 +35,13 @@
 G_DEFINE_TYPE (SwfdecAsString, swfdec_as_string, SWFDEC_TYPE_AS_OBJECT)
 
 static void
-swfdec_as_string_do_mark (SwfdecGcObject *object)
+swfdec_as_string_do_mark (SwfdecAsObject *object)
 {
   SwfdecAsString *string = SWFDEC_AS_STRING (object);
 
   swfdec_as_string_mark (string->string);
 
-  SWFDEC_GC_OBJECT_CLASS (swfdec_as_string_parent_class)->mark (object);
+  SWFDEC_AS_OBJECT_CLASS (swfdec_as_string_parent_class)->mark (object);
 }
 
 static char *
@@ -55,11 +55,9 @@ swfdec_as_string_debug (SwfdecAsObject *object)
 static void
 swfdec_as_string_class_init (SwfdecAsStringClass *klass)
 {
-  SwfdecGcObjectClass *gc_class = SWFDEC_GC_OBJECT_CLASS (klass);
   SwfdecAsObjectClass *asobject_class = SWFDEC_AS_OBJECT_CLASS (klass);
 
-  gc_class->mark = swfdec_as_string_do_mark;
-
+  asobject_class->mark = swfdec_as_string_do_mark;
   asobject_class->debug = swfdec_as_string_debug;
 }
 
@@ -338,18 +336,10 @@ void
 swfdec_as_string_valueOf (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *ret)
 {
-  if (object == NULL)
+  if (!SWFDEC_IS_AS_STRING (object))
     return;
 
-  if (SWFDEC_IS_AS_STRING (object)) {
-    SWFDEC_AS_VALUE_SET_STRING (ret, SWFDEC_AS_STRING (object)->string);
-  } else {
-    SwfdecAsValue val;
-
-    SWFDEC_AS_VALUE_SET_OBJECT (&val, object);
-    SWFDEC_AS_VALUE_SET_STRING (ret, swfdec_as_value_to_string (cx, &val));
-  }
-
+  SWFDEC_AS_VALUE_SET_STRING (ret, SWFDEC_AS_STRING (object)->string);
 }
 
 static void
@@ -361,11 +351,16 @@ swfdec_as_string_split_5 (SwfdecAsContext *cx, SwfdecAsObject *object,
   const char *str, *end, *delim;
   int count;
 
+  if (argc < 1)
+    return;
+
   str = swfdec_as_string_object_to_string (cx, object);
   arr = SWFDEC_AS_ARRAY (swfdec_as_array_new (cx));
+  if (arr == NULL)
+    return;
   SWFDEC_AS_VALUE_SET_OBJECT (ret, SWFDEC_AS_OBJECT (arr));
   /* hi, i'm the special case */
-  if (argc < 1 || SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0])) {
+  if (SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0])) {
     delim = SWFDEC_AS_STR_COMMA;
   } else {
     delim = swfdec_as_value_to_string (cx, &argv[0]);
@@ -412,21 +407,24 @@ swfdec_as_string_split_6 (SwfdecAsContext *cx, SwfdecAsObject *object,
   int count;
   guint len;
 
+  if (argc < 1)
+    return;
+
   str = swfdec_as_string_object_to_string (cx, object);
   arr = SWFDEC_AS_ARRAY (swfdec_as_array_new (cx));
+  if (arr == NULL)
+    return;
   SWFDEC_AS_VALUE_SET_OBJECT (ret, SWFDEC_AS_OBJECT (arr));
   /* hi, i'm the special case */
-  if (argc < 1 || SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0])) {
+  if (SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0])) {
     SWFDEC_AS_VALUE_SET_STRING (&val, str);
     swfdec_as_array_push (arr, &val);
     return;
   }
   delim = swfdec_as_value_to_string (cx, &argv[0]);
   if (str == SWFDEC_AS_STR_EMPTY) {
-    if (strlen (delim) > 0) {
-      SWFDEC_AS_VALUE_SET_STRING (&val, str);
-      swfdec_as_array_push (arr, &val);
-    }
+    SWFDEC_AS_VALUE_SET_STRING (&val, str);
+    swfdec_as_array_push (arr, &val);
     return;
   }
   if (argc > 1 && !SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[1]))
@@ -481,7 +479,7 @@ swfdec_as_string_slice (SwfdecAsContext *cx, SwfdecAsObject *object,
     return;
 
   str = swfdec_as_string_object_to_string (cx, object);
-  length = g_utf8_strlen (str, -1);
+  length = strlen (str);
 
   start = swfdec_as_value_to_integer (cx, &argv[0]);
   if (start < 0)
@@ -498,7 +496,7 @@ swfdec_as_string_slice (SwfdecAsContext *cx, SwfdecAsObject *object,
   }
 
   SWFDEC_AS_VALUE_SET_STRING (ret,
-      swfdec_as_str_sub (cx, str, start, end - start));
+      swfdec_as_context_give_string (cx, g_strndup (str + start, end - start)));
 }
 
 SWFDEC_AS_NATIVE (251, 7, swfdec_as_string_concat)
