@@ -1,7 +1,6 @@
 /* This is a generated file. See swfdec_as_strings.c for details. */
 #ifndef _SWFDEC_AS_STRINGS_H_
 #define _SWFDEC_AS_STRINGS_H_
-G_BEGIN_DECLS
 extern const char swfdec_as_strings[];
 #define SWFDEC_AS_STR_EMPTY &swfdec_as_strings[1]
 #define SWFDEC_AS_STR___proto__ &swfdec_as_strings[3]
@@ -491,23 +490,4 @@ extern const char swfdec_as_strings[];
 #define SWFDEC_AS_STR_lr &swfdec_as_strings[5140]
 #define SWFDEC_AS_STR_rl &swfdec_as_strings[5144]
 #define SWFDEC_AS_STR_rr &swfdec_as_strings[5148]
-#define SWFDEC_AS_STR_flash &swfdec_as_strings[5152]
-#define SWFDEC_AS_STR_geom &swfdec_as_strings[5159]
-#define SWFDEC_AS_STR_ColorTransform &swfdec_as_strings[5165]
-#define SWFDEC_AS_STR_Transform &swfdec_as_strings[5181]
-#define SWFDEC_AS_STR_bufferLength &swfdec_as_strings[5192]
-#define SWFDEC_AS_STR_bufferTime &swfdec_as_strings[5206]
-#define SWFDEC_AS_STR_audiocodec &swfdec_as_strings[5218]
-#define SWFDEC_AS_STR_currentFps &swfdec_as_strings[5230]
-#define SWFDEC_AS_STR_decodedFrames &swfdec_as_strings[5242]
-#define SWFDEC_AS_STR_liveDelay &swfdec_as_strings[5257]
-#define SWFDEC_AS_STR_videoCodec &swfdec_as_strings[5268]
-#define SWFDEC_AS_STR_System &swfdec_as_strings[5280]
-#define SWFDEC_AS_STR___resolve &swfdec_as_strings[5288]
-#define SWFDEC_AS_STR_Rectangle &swfdec_as_strings[5299]
-#define SWFDEC_AS_STR_BitmapData &swfdec_as_strings[5310]
-#define SWFDEC_AS_STR_tx &swfdec_as_strings[5322]
-#define SWFDEC_AS_STR_ty &swfdec_as_strings[5326]
-#define SWFDEC_AS_STR_Invalid_Date &swfdec_as_strings[5330]
-G_END_DECLS
 #endif
