@@ -22,8 +22,6 @@
 
 #include <swfdec/swfdec_as_native_function.h>
 
-G_BEGIN_DECLS
-
 extern const struct { 
   guint x, y; 
   SwfdecAsNative func; 
@@ -31,5 +29,4 @@ extern const struct {
   GType (* get_type) (void); 
 } native_funcs[];
 
-G_END_DECLS
 #endif
