@@ -26,8 +26,6 @@
 #include <swfdec/swfdec_color.h>
 #include <swfdec/swfdec_buffer.h>
 
-G_BEGIN_DECLS
-
 typedef struct _SwfdecBits SwfdecBits;
 
 struct _SwfdecBits
@@ -51,6 +49,7 @@ guint swfdec_bits_get_u8 (SwfdecBits * b);
 guint swfdec_bits_get_u16 (SwfdecBits * b);
 int swfdec_bits_get_s16 (SwfdecBits * b);
 guint swfdec_bits_get_u32 (SwfdecBits * b);
+int swfdec_bits_get_s32 (SwfdecBits * b);
 guint swfdec_bits_get_bu16 (SwfdecBits *b);
 guint swfdec_bits_get_bu24 (SwfdecBits *b);
 guint swfdec_bits_get_bu32 (SwfdecBits *b);
@@ -74,5 +73,4 @@ SwfdecBuffer *swfdec_bits_decompress (SwfdecBits *bits, int compressed,
     int decompressed);
 
 
-G_END_DECLS
 #endif
