@@ -44,7 +44,7 @@ swfdec_file_loader_load (SwfdecLoader *loader, SwfdecPlayer *player,
 {
   SwfdecStream *stream = SWFDEC_STREAM (loader);
   GError *error = NULL;
-  char *real, *unescape, *concat;
+  char *real;
   SwfdecURL *url;
 
   if (swfdec_url_path_is_relative (url_string)) {
@@ -68,27 +68,8 @@ swfdec_file_loader_load (SwfdecLoader *loader, SwfdecPlayer *player,
     return;
   }
 
-  unescape = g_uri_unescape_string (swfdec_url_get_path (url), NULL);
-  /* Swfdec ignores query strings, just like Flash 9.0.124.0 and onwards.
-   * Might be a useful quirk to have though */
-#ifdef QUIRKS_MODE
-  if (swfdec_url_get_query (url)) {
-    concat = g_strconcat ("/", unescape, "?", swfdec_url_get_query (url), NULL);
-  } else {
-    concat = g_strconcat ("/", unescape, NULL);
-  }
-#else
-  concat = g_strconcat ("/", unescape, NULL);
-#endif
-  g_free (unescape);
-  real = g_filename_from_utf8 (concat, -1, NULL, NULL, &error);
-  g_free (concat);
-  if (real == NULL) {
-    swfdec_stream_error (stream, "%s", error->message);
-    g_error_free (error);
-    swfdec_url_free (url);
-    return;
-  }
+  /* FIXME: append query string here? */
+  real = g_strconcat ("/", swfdec_url_get_path (url), NULL);
   buffer = swfdec_buffer_new_from_file (real, &error);
   g_free (real);
   if (buffer == NULL) {
@@ -98,7 +79,7 @@ swfdec_file_loader_load (SwfdecLoader *loader, SwfdecPlayer *player,
     swfdec_loader_set_size (loader, buffer->length);
     swfdec_stream_open (stream);
     swfdec_stream_push (stream, buffer);
-    swfdec_stream_close (stream);
+    swfdec_stream_eof (stream);
   }
   swfdec_url_free (url);
 }
