@@ -1,7 +1,7 @@
 /* Swfdec
  * Copyright (C) 2003-2006 David Schleef <ds@schleef.org>
  *		 2005-2006 Eric Anholt <eric@anholt.net>
- *		 2006-2007 Benjamin Otte <otte@gnome.org>
+ *		 2006-2008 Benjamin Otte <otte@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -30,26 +30,12 @@
 #include "jpeg.h"
 #include "swfdec_image.h"
 #include "swfdec_cache.h"
+#include "swfdec_cached_image.h"
 #include "swfdec_debug.h"
+#include "swfdec_renderer_internal.h"
 #include "swfdec_swf_decoder.h"
 
-static const cairo_user_data_key_t key;
-
-static void merge_alpha (SwfdecImage * image, unsigned char *image_data,
-    unsigned char *alpha);
-
-G_DEFINE_TYPE (SwfdecImage, swfdec_image, SWFDEC_TYPE_CACHED)
-
-static void
-swfdec_image_unload (SwfdecCached *cached)
-{
-  SwfdecImage *image = SWFDEC_IMAGE (cached);
-
-  if (image->surface) {
-    cairo_surface_destroy (image->surface);
-    image->surface = NULL;
-  }
-}
+G_DEFINE_TYPE (SwfdecImage, swfdec_image, SWFDEC_TYPE_CHARACTER)
 
 static void
 swfdec_image_dispose (GObject *object)
@@ -66,19 +52,14 @@ swfdec_image_dispose (GObject *object)
   }
 
   G_OBJECT_CLASS (swfdec_image_parent_class)->dispose (object);
-
-  g_assert (image->surface == NULL);
 }
 
 static void
 swfdec_image_class_init (SwfdecImageClass * g_class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (g_class);
-  SwfdecCachedClass *cached_class = SWFDEC_CACHED_CLASS (g_class);
 
   object_class->dispose = swfdec_image_dispose;
-
-  cached_class->unload = swfdec_image_unload;
 }
 
 static void
@@ -128,6 +109,29 @@ tag_func_define_bits_jpeg (SwfdecSwfDecoder * s, guint tag)
   return SWFDEC_STATUS_OK;
 }
 
+static gboolean
+swfdec_image_validate_size (SwfdecRenderer *renderer, guint width, guint height)
+{
+  gsize size;
+
+  if (width == 0 || height == 0)
+    return FALSE;
+
+  if (renderer) {
+    size = swfdec_renderer_get_max_cache_size (renderer);
+    size = MIN (size, G_MAXUINT);
+  } else {
+    size = G_MAXUINT;
+  }
+  if (size / 4 / width < height) {
+    SWFDEC_ERROR ("%ux%u image doesn't fit into %zu bytes of cache", 
+	width, height, size);
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
 /**
  * swfdec_jpeg_decode_argb:
  *
@@ -137,9 +141,10 @@ tag_func_define_bits_jpeg (SwfdecSwfDecoder * s, guint tag)
  * buffer.  This makes a real JPEG image out of the crap in SWF files.
  */
 static gboolean
-swfdec_jpeg_decode_argb (unsigned char *data1, int length1,
+swfdec_jpeg_decode_argb (SwfdecRenderer *renderer,
+    unsigned char *data1, int length1,
     unsigned char *data2, int length2,
-    void *outdata, int *width, int *height)
+    void *outdata, guint *width, guint *height)
 {
   gboolean ret;
 
@@ -161,37 +166,54 @@ swfdec_jpeg_decode_argb (unsigned char *data1, int length1,
     ret = FALSE;
   }
 
+  if (ret)
+    ret = swfdec_image_validate_size (renderer, *width, *height);
   return ret;
 }
 
-static void
-swfdec_image_jpeg_load (SwfdecImage *image)
+static cairo_surface_t *
+swfdec_image_create_surface_for_data (SwfdecRenderer *renderer, guint8 *data,
+    cairo_format_t format, guint width, guint height, guint rowstride)
+{
+  cairo_surface_t *surface;
+
+  if (renderer) {
+    surface = swfdec_renderer_create_for_data (renderer, data, format, width, height, rowstride);
+  } else {
+    static const cairo_user_data_key_t key;
+    surface = cairo_image_surface_create_for_data (data,
+	format, width, height, rowstride);
+    cairo_surface_set_user_data (surface, &key, data, g_free);
+  }
+  return surface;
+}
+
+static cairo_surface_t * 
+swfdec_image_jpeg_load (SwfdecImage *image, SwfdecRenderer *renderer)
 {
   gboolean ret;
   guint8 *data;
 
   if (image->jpegtables) {
-    ret = swfdec_jpeg_decode_argb (
+    ret = swfdec_jpeg_decode_argb (renderer,
         image->jpegtables->data, image->jpegtables->length,
         image->raw_data->data, image->raw_data->length,
         (void *) &data, &image->width, &image->height);
   } else {
-    ret = swfdec_jpeg_decode_argb (
+    ret = swfdec_jpeg_decode_argb (renderer,
         image->raw_data->data, image->raw_data->length,
         NULL, 0,
         (void *)&data, &image->width, &image->height);
   }
 
   if (!ret)
-    return;
-
-  swfdec_cached_load (SWFDEC_CACHED (image), 4 * image->width * image->height);
-  image->surface = cairo_image_surface_create_for_data (data, CAIRO_FORMAT_RGB24,
-      image->width, image->height, image->width * 4);
-  cairo_surface_set_user_data (image->surface, &key, data, g_free);
+    return NULL;
 
   SWFDEC_LOG ("  width = %d", image->width);
   SWFDEC_LOG ("  height = %d", image->height);
+
+  return swfdec_image_create_surface_for_data (renderer, data, 
+      CAIRO_FORMAT_RGB24, image->width, image->height, 4 * image->width);
 }
 
 int
@@ -214,25 +236,23 @@ tag_func_define_bits_jpeg_2 (SwfdecSwfDecoder * s, guint tag)
   return SWFDEC_STATUS_OK;
 }
 
-static void
-swfdec_image_jpeg2_load (SwfdecImage *image)
+static cairo_surface_t *
+swfdec_image_jpeg2_load (SwfdecImage *image, SwfdecRenderer *renderer)
 {
   gboolean ret;
   guint8 *data;
 
-  ret = swfdec_jpeg_decode_argb (image->raw_data->data, image->raw_data->length,
+  ret = swfdec_jpeg_decode_argb (renderer, image->raw_data->data, image->raw_data->length,
       NULL, 0,
       (void *)&data, &image->width, &image->height);
   if (!ret)
-    return;
-
-  swfdec_cached_load (SWFDEC_CACHED (image), 4 * image->width * image->height);
-  image->surface = cairo_image_surface_create_for_data (data, CAIRO_FORMAT_RGB24,
-      image->width, image->height, image->width * 4);
-  cairo_surface_set_user_data (image->surface, &key, data, g_free);
+    return NULL;
 
   SWFDEC_LOG ("  width = %d", image->width);
   SWFDEC_LOG ("  height = %d", image->height);
+
+  return swfdec_image_create_surface_for_data (renderer, data, 
+      CAIRO_FORMAT_RGB24, image->width, image->height, 4 * image->width);
 }
 
 int
@@ -255,53 +275,11 @@ tag_func_define_bits_jpeg_3 (SwfdecSwfDecoder * s, guint tag)
   return SWFDEC_STATUS_OK;
 }
 
-static void
-swfdec_image_jpeg3_load (SwfdecImage *image)
-{
-  SwfdecBits bits;
-  SwfdecBuffer *buffer;
-  int jpeg_length;
-  gboolean ret;
-  guint8 *data;
-
-  swfdec_bits_init (&bits, image->raw_data);
-
-  jpeg_length = swfdec_bits_get_u32 (&bits);
-  buffer = swfdec_bits_get_buffer (&bits, jpeg_length);
-  if (buffer == NULL)
-    return;
-
-  ret = swfdec_jpeg_decode_argb (buffer->data, buffer->length,
-      NULL, 0,
-      (void *)&data, &image->width, &image->height);
-  swfdec_buffer_unref (buffer);
-
-  if (!ret)
-    return;
-
-  swfdec_cached_load (SWFDEC_CACHED (image), 4 * image->width * image->height);
-
-  buffer = swfdec_bits_decompress (&bits, -1, image->width * image->height);
-  if (buffer) {
-    merge_alpha (image, data, buffer->data);
-    swfdec_buffer_unref (buffer);
-  } else {
-    SWFDEC_WARNING ("cannot set alpha channel information, decompression failed");
-  }
-
-  SWFDEC_LOG ("  width = %d", image->width);
-  SWFDEC_LOG ("  height = %d", image->height);
-
-  image->surface = cairo_image_surface_create_for_data (data,
-      CAIRO_FORMAT_ARGB32, image->width, image->height, image->width * 4);
-  cairo_surface_set_user_data (image->surface, &key, data, g_free);
-}
-
 static void
 merge_alpha (SwfdecImage * image, unsigned char *image_data,
     unsigned char *alpha)
 {
-  int x, y;
+  unsigned int x, y;
   unsigned char *p;
 
   for (y = 0; y < image->height; y++) {
@@ -314,8 +292,47 @@ merge_alpha (SwfdecImage * image, unsigned char *image_data,
   }
 }
 
-static void
-swfdec_image_lossless_load (SwfdecImage *image)
+static cairo_surface_t *
+swfdec_image_jpeg3_load (SwfdecImage *image, SwfdecRenderer *renderer)
+{
+  SwfdecBits bits;
+  SwfdecBuffer *buffer;
+  int jpeg_length;
+  gboolean ret;
+  guint8 *data;
+
+  swfdec_bits_init (&bits, image->raw_data);
+
+  jpeg_length = swfdec_bits_get_u32 (&bits);
+  buffer = swfdec_bits_get_buffer (&bits, jpeg_length);
+  if (buffer == NULL)
+    return NULL;
+
+  ret = swfdec_jpeg_decode_argb (renderer,
+      buffer->data, buffer->length, NULL, 0,
+      (void *)&data, &image->width, &image->height);
+  swfdec_buffer_unref (buffer);
+
+  if (!ret)
+    return NULL;
+
+  buffer = swfdec_bits_decompress (&bits, -1, image->width * image->height);
+  if (buffer) {
+    merge_alpha (image, data, buffer->data);
+    swfdec_buffer_unref (buffer);
+  } else {
+    SWFDEC_WARNING ("cannot set alpha channel information, decompression failed");
+  }
+
+  SWFDEC_LOG ("  width = %d", image->width);
+  SWFDEC_LOG ("  height = %d", image->height);
+
+  return swfdec_image_create_surface_for_data (renderer, data, 
+      CAIRO_FORMAT_ARGB32, image->width, image->height, 4 * image->width);
+}
+
+static cairo_surface_t *
+swfdec_image_lossless_load (SwfdecImage *image, SwfdecRenderer *renderer)
 {
   int format;
   unsigned char *ptr;
@@ -336,12 +353,8 @@ swfdec_image_lossless_load (SwfdecImage *image)
   SWFDEC_LOG ("width = %d", image->width);
   SWFDEC_LOG ("height = %d", image->height);
 
-  if (image->width == 0 || image->height == 0)
-    return;
-  if (G_MAXUINT / 4 / image->width < (guint) image->height)
-    return;
-
-  swfdec_cached_load (SWFDEC_CACHED (image), 4 * image->width * image->height);
+  if (!swfdec_image_validate_size (renderer, image->width, image->height))
+    return NULL;
 
   if (format == 3) {
     SwfdecBuffer *buffer;
@@ -397,7 +410,7 @@ swfdec_image_lossless_load (SwfdecImage *image)
 
     swfdec_buffer_unref (buffer);
   } else if (format == 4) {
-    int i, j;
+    guint i, j;
     guint c;
     unsigned char *idata;
     SwfdecBuffer *buffer;
@@ -434,7 +447,7 @@ swfdec_image_lossless_load (SwfdecImage *image)
     swfdec_buffer_unref (buffer);
   } else if (format == 5) {
     SwfdecBuffer *buffer;
-    int i, j;
+    guint i, j;
     guint32 *p;
 
     buffer = swfdec_bits_decompress (&bits, -1, 4 * image->width * image->height);
@@ -452,22 +465,16 @@ swfdec_image_lossless_load (SwfdecImage *image)
 	p++;
       }
     }
-    image->surface = cairo_image_surface_create_for_data (data, 
-	have_alpha ? CAIRO_FORMAT_ARGB32 : CAIRO_FORMAT_RGB24, 
-	image->width, image->height, image->width * 4);
-    cairo_surface_set_user_data (image->surface, &key, buffer, 
-	(cairo_destroy_func_t) swfdec_buffer_unref);
-    return;
+    data = g_memdup (buffer->data, buffer->length);
   } else {
     SWFDEC_ERROR ("unknown lossless image format %u", format);
-    return;
+    return NULL;
   }
 
 out:
-  image->surface = cairo_image_surface_create_for_data (data, 
+  return swfdec_image_create_surface_for_data (renderer, data,
       have_alpha ? CAIRO_FORMAT_ARGB32 : CAIRO_FORMAT_RGB24, 
       image->width, image->height, image->width * 4);
-  cairo_surface_set_user_data (image->surface, &key, data, g_free);
 }
 
 int
@@ -524,64 +531,132 @@ swfdec_image_png_read (void *bitsp, unsigned char *data, unsigned int length)
   return CAIRO_STATUS_SUCCESS;
 }
 
-static void
-swfdec_image_png_load (SwfdecImage *image)
+static cairo_surface_t *
+swfdec_image_png_load (SwfdecImage *image, SwfdecRenderer *renderer)
 {
   SwfdecBits bits;
+  cairo_surface_t *surface;
 
   swfdec_bits_init (&bits, image->raw_data);
-  image->surface = cairo_image_surface_create_from_png_stream (
+  surface = cairo_image_surface_create_from_png_stream (
       swfdec_image_png_read, &bits);
-  image->width = cairo_image_surface_get_width (image->surface);
-  image->height = cairo_image_surface_get_height (image->surface);
-  swfdec_cached_load (SWFDEC_CACHED (image), image->height *
-      cairo_image_surface_get_stride (image->surface));
+  if (cairo_surface_status (surface) != CAIRO_STATUS_SUCCESS) {
+    SWFDEC_ERROR ("could not create PNG image: %s", 
+	cairo_status_to_string (cairo_surface_status (surface)));
+    cairo_surface_destroy (surface);
+    return NULL;
+  }
+
+  image->width = cairo_image_surface_get_width (surface);
+  image->height = cairo_image_surface_get_height (surface);
+  if (!swfdec_image_validate_size (renderer, image->width, image->height)) {
+    cairo_surface_destroy (surface);
+    return NULL;
+  }
+  if (renderer)
+    surface = swfdec_renderer_create_similar (renderer, surface);
+  return surface;
+}
+
+static gboolean
+swfdec_image_find_by_transform (SwfdecCached *cached, gpointer data)
+{
+  const SwfdecColorTransform *trans = data;
+  SwfdecColorTransform ctrans;
+
+  swfdec_cached_image_get_color_transform (SWFDEC_CACHED_IMAGE (cached),
+      &ctrans);
+  if (trans->mask != ctrans.mask)
+    return FALSE;
+  if (trans->mask)
+    return TRUE;
+  return (trans->ra == ctrans.ra && 
+      trans->rb == ctrans.rb && 
+      trans->ga == ctrans.ga && 
+      trans->gb == ctrans.gb && 
+      trans->ba == ctrans.ba && 
+      trans->bb == ctrans.bb && 
+      trans->aa == ctrans.aa && 
+      trans->ab == ctrans.ab);
+}
+
+static cairo_surface_t *
+swfdec_image_lookup_surface (SwfdecImage *image, SwfdecRenderer *renderer,
+    const SwfdecColorTransform *trans)
+{
+  if (renderer) {
+    SwfdecCached *cached = swfdec_renderer_get_cache (renderer, image, 
+	swfdec_image_find_by_transform, (gpointer) trans);
+    if (cached) {
+      swfdec_cached_use (cached);
+      return swfdec_cached_image_get_surface (SWFDEC_CACHED_IMAGE (cached));
+    }
+  }
+  return NULL;
 }
 
 cairo_surface_t *
-swfdec_image_create_surface (SwfdecImage *image)
+swfdec_image_create_surface (SwfdecImage *image, SwfdecRenderer *renderer)
 {
+  SwfdecColorTransform trans;
+  SwfdecCachedImage *cached;
+  cairo_surface_t *surface;
+
+  g_return_val_if_fail (SWFDEC_IS_IMAGE (image), NULL);
+  g_return_val_if_fail (renderer == NULL || SWFDEC_IS_RENDERER (renderer), NULL);
+
   if (image->raw_data == NULL)
     return NULL;
 
-  if (image->surface == NULL) {
-    switch (image->type) {
-      case SWFDEC_IMAGE_TYPE_JPEG:
-	swfdec_image_jpeg_load (image);
-	break;
-      case SWFDEC_IMAGE_TYPE_JPEG2:
-	swfdec_image_jpeg2_load (image);
-	break;
-      case SWFDEC_IMAGE_TYPE_JPEG3:
-	swfdec_image_jpeg3_load (image);
-	break;
-      case SWFDEC_IMAGE_TYPE_LOSSLESS:
-	swfdec_image_lossless_load (image);
-	break;
-      case SWFDEC_IMAGE_TYPE_LOSSLESS2:
-	swfdec_image_lossless_load (image);
-	break;
-      case SWFDEC_IMAGE_TYPE_PNG:
-	swfdec_image_png_load (image);
-	break;
-      case SWFDEC_IMAGE_TYPE_UNKNOWN:
-      default:
-	g_assert_not_reached ();
-	break;
-    }
-    if (image->surface == NULL) {
-      SWFDEC_WARNING ("failed to decode image");
-      return NULL;
-    }
-  } else {
-    swfdec_cached_use (SWFDEC_CACHED (image));
+  swfdec_color_transform_init_identity (&trans);
+  surface = swfdec_image_lookup_surface (image, renderer, &trans);
+  if (surface)
+    return surface;
+
+  switch (image->type) {
+    case SWFDEC_IMAGE_TYPE_JPEG:
+      surface = swfdec_image_jpeg_load (image, renderer);
+      break;
+    case SWFDEC_IMAGE_TYPE_JPEG2:
+      surface = swfdec_image_jpeg2_load (image, renderer);
+      break;
+    case SWFDEC_IMAGE_TYPE_JPEG3:
+      surface = swfdec_image_jpeg3_load (image, renderer);
+      break;
+    case SWFDEC_IMAGE_TYPE_LOSSLESS:
+      surface = swfdec_image_lossless_load (image, renderer);
+      break;
+    case SWFDEC_IMAGE_TYPE_LOSSLESS2:
+      surface = swfdec_image_lossless_load (image, renderer);
+      break;
+    case SWFDEC_IMAGE_TYPE_PNG:
+      surface = swfdec_image_png_load (image, renderer);
+      break;
+    case SWFDEC_IMAGE_TYPE_UNKNOWN:
+    default:
+      g_assert_not_reached ();
+      break;
   }
-  return cairo_surface_reference (image->surface);
+  if (surface == NULL) {
+    SWFDEC_WARNING ("failed to decode image");
+    return NULL;
+  }
+  if (renderer) {
+    /* FIXME: The size is just an educated guess */
+    cached = swfdec_cached_image_new (surface, image->width * image->height * 4);
+    swfdec_renderer_add_cache (renderer, FALSE, image, SWFDEC_CACHED (cached));
+    g_object_unref (cached);
+  }
+
+  return surface;
 }
 
 cairo_surface_t *
-swfdec_image_create_surface_transformed (SwfdecImage *image, const SwfdecColorTransform *trans)
+swfdec_image_create_surface_transformed (SwfdecImage *image, SwfdecRenderer *renderer,
+    const SwfdecColorTransform *trans)
 {
+  SwfdecColorTransform mask;
+  SwfdecCachedImage *cached;
   cairo_surface_t *surface, *source;
   guint32 *tdata;
   const guint32 *sdata;
@@ -589,13 +664,35 @@ swfdec_image_create_surface_transformed (SwfdecImage *image, const SwfdecColorTr
   gboolean has_alpha = FALSE;
 
   g_return_val_if_fail (SWFDEC_IS_IMAGE (image), NULL);
+  g_return_val_if_fail (renderer == NULL || SWFDEC_IS_RENDERER (renderer), NULL);
   g_return_val_if_fail (trans != NULL, NULL);
+  /* The mask flag is used for caching image surfaces.
+   * Instead of masking with images, code should use color patterns */
+  g_return_val_if_fail (!swfdec_color_transform_is_mask (trans), NULL);
 
+  surface = swfdec_image_lookup_surface (image, renderer, trans);
+  if (surface)
+    return surface;
   /* obvious optimization */
   if (swfdec_color_transform_is_identity (trans))
-    return swfdec_image_create_surface (image);
+    return swfdec_image_create_surface (image, renderer);
+
+  /* need to create an image surface here, so we can modify it. Will upload later */
+  /* NB: we use the mask property here to inidicate an image surface */
+  swfdec_color_transform_init_mask (&mask);
+  source = swfdec_image_lookup_surface (image, renderer, &mask);
+  if (source == NULL) {
+    source = swfdec_image_create_surface (image, NULL);
+    if (source == NULL)
+      return NULL;
+    if (renderer) {
+      cached = swfdec_cached_image_new (source, image->width * image->height * 4);
+      swfdec_cached_image_set_color_transform (cached, &mask);
+      swfdec_renderer_add_cache (renderer, FALSE, image, SWFDEC_CACHED (cached));
+      g_object_unref (cached);
+    }
+  }
 
-  source = swfdec_image_create_surface (image);
   tdata = g_try_malloc (image->width * image->height * 4);
   if (!tdata) {
     SWFDEC_ERROR ("failed to allocate memory for transformed image");
@@ -612,10 +709,16 @@ swfdec_image_create_surface_transformed (SwfdecImage *image, const SwfdecColorTr
     has_alpha |= SWFDEC_COLOR_A (tdata[i]) != 0xFF;
   }
   cairo_surface_destroy (source);
-  surface = cairo_image_surface_create_for_data ((unsigned char *) tdata,
+  surface = swfdec_image_create_surface_for_data (renderer, (guchar *) tdata,
       has_alpha ? CAIRO_FORMAT_ARGB32 : CAIRO_FORMAT_RGB24, 
       image->width, image->height, image->width * 4);
-  cairo_surface_set_user_data (surface, &key, tdata, g_free);
+  if (renderer) {
+    /* FIXME: The size is just an educated guess */
+    cached = swfdec_cached_image_new (surface, image->width * image->height * 4);
+    swfdec_cached_image_set_color_transform (cached, trans);
+    swfdec_renderer_add_cache (renderer, FALSE, image, SWFDEC_CACHED (cached));
+    g_object_unref (cached);
+  }
   return surface;
 }
 
