@@ -91,7 +91,6 @@ swfdec_load_object_as_load (SwfdecAsContext *cx, SwfdecAsObject *object, guint a
   SWFDEC_AS_VALUE_SET_BOOLEAN (rval, TRUE);
 }
 
-#define ALLOWED_CHARACTERS " !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~"
 static void
 swfdec_load_object_as_get_headers (SwfdecAsObject *object, guint *header_count,
     char ***header_names, char ***header_values)
@@ -112,7 +111,7 @@ swfdec_load_object_as_get_headers (SwfdecAsObject *object, guint *header_count,
   const char *name;
   SwfdecAsContext *cx;
 
-  cx = swfdec_gc_object_get_context (object);
+  cx = object->context;
 
   array_names = g_ptr_array_new ();
   array_values = g_ptr_array_new ();
@@ -144,20 +143,14 @@ swfdec_load_object_as_get_headers (SwfdecAsObject *object, guint *header_count,
     if (name == NULL) {
       name = swfdec_as_value_to_string (cx, &val);
     } else {
-      const char *value = swfdec_as_value_to_string (cx, &val);
       for (j = 0; j < G_N_ELEMENTS (disallowed_names); j++) {
 	if (g_ascii_strcasecmp (name, disallowed_names[j]) == 0)
 	  break;
       }
-      if (j < G_N_ELEMENTS (disallowed_names)) {
-	SWFDEC_WARNING ("Custom header with name %s is not allowed", name);
-      } else if (strspn (name, ALLOWED_CHARACTERS) != strlen (name) || strchr (name, ':') != NULL || strchr (name, ' ') != NULL) {
-	SWFDEC_WARNING ("Custom header's name (%s) contains characters that are not allowed", name);
-      } else if (strspn (value, ALLOWED_CHARACTERS) != strlen (value)) {
-	SWFDEC_WARNING ("Custom header's value (%s) contains characters that are not allowed", value);
-      } else {
+      if (j >= G_N_ELEMENTS (disallowed_names)) {
 	g_ptr_array_add (array_names, g_strdup (name));
-	g_ptr_array_add (array_values, g_strdup (value));
+	g_ptr_array_add (array_values,
+	    g_strdup (swfdec_as_value_to_string (cx, &val)));
       }
       name = NULL;
     }
@@ -174,7 +167,6 @@ end:
   *header_names = (char **)g_ptr_array_free (array_names, FALSE);
   *header_values = (char **)g_ptr_array_free (array_values, FALSE);
 }
-#undef ALLOWED_CHARACTERS
 
 SWFDEC_AS_NATIVE (301, 1, swfdec_load_object_as_send)
 void
@@ -182,8 +174,6 @@ swfdec_load_object_as_send (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *rval)
 {
   const char *url, *target, *method, *data;
-  guint header_count;
-  char **header_names, **header_values;
   SwfdecAsValue val;
   SwfdecBuffer *buffer;
 
@@ -203,10 +193,9 @@ swfdec_load_object_as_send (SwfdecAsContext *cx, SwfdecAsObject *object,
 	strlen (data));
   }
 
-  swfdec_load_object_as_get_headers (object, &header_count, &header_names,
-      &header_values);
-  swfdec_player_launch_with_headers (SWFDEC_PLAYER (cx), url, target, buffer,
-      header_count, header_names, header_values);
+  SWFDEC_FIXME ("XML/LoadVars.send: Support for sending headers missing");
+
+  swfdec_player_launch (SWFDEC_PLAYER (cx), url, target, buffer);
 
   SWFDEC_AS_VALUE_SET_BOOLEAN (rval, TRUE);
 }
