@@ -134,24 +134,25 @@ swfdec_marshal_BOOLEAN__UINT_UINT_BOOLEAN (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* VOID:POINTER,UINT (./swfdec_marshal.list:3) */
+/* VOID:BOXED,POINTER,UINT (./swfdec_marshal.list:3) */
 void
-swfdec_marshal_VOID__POINTER_UINT (GClosure     *closure,
-                                   GValue       *return_value G_GNUC_UNUSED,
-                                   guint         n_param_values,
-                                   const GValue *param_values,
-                                   gpointer      invocation_hint G_GNUC_UNUSED,
-                                   gpointer      marshal_data)
+swfdec_marshal_VOID__BOXED_POINTER_UINT (GClosure     *closure,
+                                         GValue       *return_value G_GNUC_UNUSED,
+                                         guint         n_param_values,
+                                         const GValue *param_values,
+                                         gpointer      invocation_hint G_GNUC_UNUSED,
+                                         gpointer      marshal_data)
 {
-  typedef void (*GMarshalFunc_VOID__POINTER_UINT) (gpointer     data1,
-                                                   gpointer     arg_1,
-                                                   guint        arg_2,
-                                                   gpointer     data2);
-  register GMarshalFunc_VOID__POINTER_UINT callback;
+  typedef void (*GMarshalFunc_VOID__BOXED_POINTER_UINT) (gpointer     data1,
+                                                         gpointer     arg_1,
+                                                         gpointer     arg_2,
+                                                         guint        arg_3,
+                                                         gpointer     data2);
+  register GMarshalFunc_VOID__BOXED_POINTER_UINT callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
-  g_return_if_fail (n_param_values == 3);
+  g_return_if_fail (n_param_values == 4);
 
   if (G_CCLOSURE_SWAP_DATA (closure))
     {
@@ -163,11 +164,12 @@ swfdec_marshal_VOID__POINTER_UINT (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__POINTER_UINT) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__BOXED_POINTER_UINT) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
-            g_marshal_value_peek_pointer (param_values + 1),
-            g_marshal_value_peek_uint (param_values + 2),
+            g_marshal_value_peek_boxed (param_values + 1),
+            g_marshal_value_peek_pointer (param_values + 2),
+            g_marshal_value_peek_uint (param_values + 3),
             data2);
 }
 
