@@ -52,11 +52,7 @@
 
 enum {
   PROP_0,
-  PROP_DEPTH,
-  PROP_GRAPHIC,
-  PROP_NAME,
-  PROP_PARENT,
-  PROP_RESOURCE
+  PROP_DEPTH
 };
 
 enum {
@@ -80,6 +76,7 @@ swfdec_movie_init (SwfdecMovie * movie)
   cairo_matrix_init_identity (&movie->inverse_matrix);
 
   swfdec_color_transform_init_identity (&movie->color_transform);
+  swfdec_color_transform_init_identity (&movie->original_ctrans);
 
   movie->visible = TRUE;
   movie->cache_state = SWFDEC_MOVIE_INVALID_EXTENTS;
@@ -115,7 +112,7 @@ swfdec_movie_invalidate (SwfdecMovie *movie, const cairo_matrix_t *parent_to_glo
     if (movie->invalidate_last)
       return;
     movie->invalidate_last = TRUE;
-    player = SWFDEC_PLAYER (swfdec_gc_object_get_context (movie));
+    player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
     player->priv->invalid_pending = g_slist_prepend (player->priv->invalid_pending, movie);
   }
   g_assert (movie->cache_state <= SWFDEC_MOVIE_INVALID_CHILDREN);
@@ -168,7 +165,7 @@ swfdec_movie_invalidate_next (SwfdecMovie *movie)
 
   swfdec_movie_invalidate_last (movie);
   movie->invalidate_next = TRUE;
-  player = SWFDEC_PLAYER (swfdec_gc_object_get_context (movie));
+  player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
   if (movie == SWFDEC_MOVIE (player->priv->focus))
     swfdec_player_invalidate_focusrect (player);
 }
@@ -353,7 +350,7 @@ swfdec_movie_do_remove (SwfdecMovie *movie, gboolean destroy)
 
   SWFDEC_LOG ("removing %s %s", G_OBJECT_TYPE_NAME (movie), movie->name);
 
-  player = SWFDEC_PLAYER (swfdec_gc_object_get_context (movie));
+  player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
   while (movie->list) {
     GList *walk = movie->list;
     while (walk && SWFDEC_MOVIE (walk->data)->state >= SWFDEC_MOVIE_STATE_REMOVED)
@@ -414,7 +411,7 @@ void
 swfdec_movie_destroy (SwfdecMovie *movie)
 {
   SwfdecMovieClass *klass = SWFDEC_MOVIE_GET_CLASS (movie);
-  SwfdecPlayer *player = SWFDEC_PLAYER (swfdec_gc_object_get_context (movie));
+  SwfdecPlayer *player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
 
   g_assert (movie->state < SWFDEC_MOVIE_STATE_DESTROYED);
   SWFDEC_LOG ("destroying movie %s", movie->name);
@@ -594,7 +591,7 @@ swfdec_movie_get_mouse (SwfdecMovie *movie, double *x, double *y)
   g_return_if_fail (x != NULL);
   g_return_if_fail (y != NULL);
 
-  player = SWFDEC_PLAYER (swfdec_gc_object_get_context (movie));
+  player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
   *x = player->priv->mouse_x;
   *y = player->priv->mouse_y;
   swfdec_player_stage_to_global (player, x, y);
@@ -734,11 +731,12 @@ swfdec_movie_get_operator_for_blend_mode (guint blend_mode)
  * @cr: a cairo context which should be used for masking. The cairo context's
  *      matrix is assumed to be in the coordinate system of the movie's parent.
  * @matrix: matrix to apply before rendering
+ * @inval: The region that is relevant for masking
  *
  * Creates a pattern suitable for masking. To do rendering using the returned
  * mask, you want to use code like this:
  * <informalexample><programlisting>
- * mask = swfdec_movie_mask (cr, movie, matrix);
+ * mask = swfdec_movie_mask (cr, movie, matrix, inval);
  * cairo_push_group (cr);
  * // do rendering here
  * cairo_pop_group_to_source (cr);
@@ -750,24 +748,35 @@ swfdec_movie_get_operator_for_blend_mode (guint blend_mode)
  **/
 static cairo_pattern_t *
 swfdec_movie_mask (cairo_t *cr, SwfdecMovie *movie,
-    const cairo_matrix_t *matrix)
+    const cairo_matrix_t *matrix, const SwfdecRect *inval)
 {
   SwfdecColorTransform black;
+  cairo_matrix_t inv;
+  SwfdecRect rect;
 
   swfdec_color_transform_init_mask (&black);
   cairo_push_group_with_content (cr, CAIRO_CONTENT_ALPHA);
   cairo_transform (cr, matrix);
+  inv = *matrix;
+  if (cairo_matrix_invert (&inv) == CAIRO_STATUS_SUCCESS && FALSE) {
+    swfdec_rect_transform (&rect, inval, &inv);
+  } else {
+    SWFDEC_INFO ("non-invertible matrix when computing invalid area");
+    rect.x0 = rect.y0 = -G_MAXDOUBLE;
+    rect.x1 = rect.y1 = G_MAXDOUBLE;
+  }
 
-  swfdec_movie_render (movie, cr, &black);
+  swfdec_movie_render (movie, cr, &black, &rect);
   return cairo_pop_group (cr);
 }
 
 void
 swfdec_movie_render (SwfdecMovie *movie, cairo_t *cr,
-    const SwfdecColorTransform *color_transform)
+    const SwfdecColorTransform *color_transform, const SwfdecRect *inval)
 {
   SwfdecMovieClass *klass;
   SwfdecColorTransform trans;
+  SwfdecRect rect;
   gboolean group;
 
   g_return_if_fail (SWFDEC_IS_MOVIE (movie));
@@ -776,12 +785,25 @@ swfdec_movie_render (SwfdecMovie *movie, cairo_t *cr,
     g_warning ("%s", cairo_status_to_string (cairo_status (cr)));
   }
   g_return_if_fail (color_transform != NULL);
+  g_return_if_fail (inval != NULL);
   
   if (movie->mask_of != NULL && !swfdec_color_transform_is_mask (color_transform)) {
     SWFDEC_LOG ("not rendering %s %p, movie is a mask",
 	G_OBJECT_TYPE_NAME (movie), movie->name);
     return;
   }
+  if (!swfdec_rect_intersect (NULL, &movie->extents, inval)) {
+    SWFDEC_LOG ("not rendering %s %s, extents %g %g  %g %g are not in invalid area %g %g  %g %g",
+	G_OBJECT_TYPE_NAME (movie), movie->name, 
+	movie->extents.x0, movie->extents.y0, movie->extents.x1, movie->extents.y1,
+	inval->x0, inval->y0, inval->x1, inval->y1);
+    return;
+  }
+  if (!movie->visible) {
+    SWFDEC_LOG ("not rendering %s %p, movie is invisible",
+	G_OBJECT_TYPE_NAME (movie), movie->name);
+    return;
+  }
 
   if (movie->masked_by != NULL) {
     cairo_push_group (cr);
@@ -798,11 +820,15 @@ swfdec_movie_render (SwfdecMovie *movie, cairo_t *cr,
       movie->matrix.xy, movie->matrix.yx,
       movie->matrix.x0, movie->matrix.y0);
   cairo_transform (cr, &movie->matrix);
-  swfdec_color_transform_chain (&trans, &movie->color_transform, color_transform);
+  swfdec_rect_transform (&rect, inval, &movie->inverse_matrix);
+  SWFDEC_LOG ("%sinvalid area is now: %g %g  %g %g",  movie->parent ? "  " : "",
+      rect.x0, rect.y0, rect.x1, rect.y1);
+  swfdec_color_transform_chain (&trans, &movie->original_ctrans, color_transform);
+  swfdec_color_transform_chain (&trans, &movie->color_transform, &trans);
 
   klass = SWFDEC_MOVIE_GET_CLASS (movie);
   g_return_if_fail (klass->render);
-  klass->render (movie, cr, &trans);
+  klass->render (movie, cr, &trans, &rect);
 #if 0
   /* code to draw a red rectangle around the area occupied by this movie clip */
   {
@@ -842,7 +868,7 @@ swfdec_movie_render (SwfdecMovie *movie, cairo_t *cr,
       swfdec_movie_local_to_global_matrix (movie->masked_by->parent, &mat2);
       cairo_matrix_multiply (&mat, &mat, &mat2);
     }
-    mask = swfdec_movie_mask (cr, movie->masked_by, &mat);
+    mask = swfdec_movie_mask (cr, movie->masked_by, &mat, inval);
     cairo_pop_group_to_source (cr);
     cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
     cairo_mask (cr, mask);
@@ -860,12 +886,6 @@ swfdec_movie_get_property (GObject *object, guint param_id, GValue *value,
     case PROP_DEPTH:
       g_value_set_int (value, movie->depth);
       break;
-    case PROP_PARENT:
-      g_value_set_object (value, movie->parent);
-      break;
-    case PROP_RESOURCE:
-      g_value_set_object (value, movie->resource);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
       break;
@@ -877,63 +897,11 @@ swfdec_movie_set_property (GObject *object, guint param_id, const GValue *value,
     GParamSpec * pspec)
 {
   SwfdecMovie *movie = SWFDEC_MOVIE (object);
-  SwfdecAsContext *cx = swfdec_gc_object_get_context (movie);
-
-  /* The context must be set before all movie-related properties */
-  g_assert (cx);
 
   switch (param_id) {
     case PROP_DEPTH:
-      /* parent must be set after depth */
-      g_assert (movie->parent == NULL);
       movie->depth = g_value_get_int (value);
       break;
-    case PROP_GRAPHIC:
-      movie->graphic = g_value_get_object (value);
-      if (movie->graphic)
-	g_object_ref (movie->graphic);
-      break;
-    case PROP_NAME:
-      movie->name = g_value_get_string (value);
-      if (movie->name) {
-	movie->name = swfdec_as_context_get_string (cx, movie->name);
-	movie->original_name = movie->name;
-      } else {
-	movie->original_name = SWFDEC_AS_STR_EMPTY;
-	if (SWFDEC_IS_SPRITE_MOVIE (movie) || SWFDEC_IS_BUTTON_MOVIE (movie)) {
-	  movie->name = swfdec_as_context_give_string (cx,
-	      g_strdup_printf ("instance%u", ++SWFDEC_PLAYER (cx)->priv->unnamed_count));
-	} else {
-	  movie->name = SWFDEC_AS_STR_EMPTY;
-	}
-      }
-      break;
-    case PROP_PARENT:
-      movie->parent = g_value_get_object (value);
-      /* parent holds a reference */
-      g_object_ref (movie);
-      if (movie->parent) {
-	movie->parent->list = g_list_insert_sorted (movie->parent->list, movie, swfdec_movie_compare_depths);
-	SWFDEC_DEBUG ("inserting %s %p into %s %p", G_OBJECT_TYPE_NAME (movie), movie,
-	    G_OBJECT_TYPE_NAME (movie->parent), movie->parent);
-	/* invalidate the parent, so it gets visible */
-	swfdec_movie_queue_update (movie->parent, SWFDEC_MOVIE_INVALID_CHILDREN);
-      } else {
-	SwfdecAsValue val;
-	SwfdecPlayerPrivate *priv = SWFDEC_PLAYER (cx)->priv;
-	priv->roots = g_list_insert_sorted (priv->roots, movie, swfdec_movie_compare_depths);
-	SWFDEC_AS_VALUE_SET_STRING (&val, swfdec_as_context_get_string (cx, priv->system->version));
-	swfdec_as_object_set_variable (SWFDEC_AS_OBJECT (movie), SWFDEC_AS_STR__version, &val);
-      }
-      break;
-    case PROP_RESOURCE:
-      movie->resource = g_value_get_object (value);
-      /* NB: the resource assumes it can access the player via the movie */
-      if (movie->resource->movie == NULL) {
-	g_assert (SWFDEC_IS_SPRITE_MOVIE (movie));
-	movie->resource->movie = SWFDEC_SPRITE_MOVIE (movie);
-      }
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
       break;
@@ -971,46 +939,32 @@ swfdec_movie_dispose (GObject *object)
 }
 
 static void
-swfdec_movie_mark (SwfdecGcObject *object)
+swfdec_movie_mark (SwfdecAsObject *object)
 {
   SwfdecMovie *movie = SWFDEC_MOVIE (object);
   GList *walk;
   GSList *iter;
 
   if (movie->parent)
-    swfdec_gc_object_mark (movie->parent);
+    swfdec_as_object_mark (SWFDEC_AS_OBJECT (movie->parent));
   swfdec_as_string_mark (movie->original_name);
   swfdec_as_string_mark (movie->name);
   for (walk = movie->list; walk; walk = walk->next) {
-    swfdec_gc_object_mark (walk->data);
+    swfdec_as_object_mark (walk->data);
   }
   for (iter = movie->variable_listeners; iter != NULL; iter = iter->next) {
     SwfdecMovieVariableListener *listener = iter->data;
-    swfdec_gc_object_mark (listener->object);
+    swfdec_as_object_mark (listener->object);
     swfdec_as_string_mark (listener->name);
   }
-  swfdec_gc_object_mark (movie->resource);
+  swfdec_as_object_mark (SWFDEC_AS_OBJECT (movie->resource));
 
-  SWFDEC_GC_OBJECT_CLASS (swfdec_movie_parent_class)->mark (object);
+  SWFDEC_AS_OBJECT_CLASS (swfdec_movie_parent_class)->mark (object);
 }
 
-/**
- * swfdec_movie_is_scriptable:
- * @movie: a movie
- *
- * Checks if the movie may be accessed by scripts. If not, the movie is not
- * accessible by Actionscript and functions that would return the movie should
- * instead return its parent.
- *
- * Returns: %TRUE if scripts may access this movie, %FALSE if the parent 
- *          should be used.
- **/
-gboolean
-swfdec_movie_is_scriptable (SwfdecMovie *movie)
-{
-  return (SWFDEC_IS_ACTOR (movie) || SWFDEC_IS_VIDEO_MOVIE (movie)) &&
-   (swfdec_movie_get_version (movie) > 5 || !SWFDEC_IS_TEXT_FIELD_MOVIE (movie));
-}
+#define swfdec_movie_is_scriptable(mov) \
+  ((SWFDEC_IS_ACTOR (mov) || SWFDEC_IS_VIDEO_MOVIE (mov)) && \
+   (swfdec_movie_get_version (mov) > 5 || !SWFDEC_IS_TEXT_FIELD_MOVIE (mov)))
 
 /* FIXME: This function can definitely be implemented easier */
 SwfdecMovie *
@@ -1018,12 +972,27 @@ swfdec_movie_get_by_name (SwfdecMovie *movie, const char *name, gboolean unnamed
 {
   GList *walk;
   int i;
-  guint version = swfdec_gc_object_get_context (movie)->version;
-  SwfdecPlayer *player = SWFDEC_PLAYER (swfdec_gc_object_get_context (movie));
+  gulong l;
+  guint version = SWFDEC_AS_OBJECT (movie)->context->version;
+  char *end;
+  SwfdecPlayer *player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context);
 
-  i = swfdec_player_get_level (player, name, version);
-  if (i >= 0)
-    return SWFDEC_MOVIE (swfdec_player_get_movie_at_level (player, i));
+  if ((version >= 7 && g_str_has_prefix (name, "_level")) ||
+      (version < 7 && strncasecmp (name, "_level", 6) == 0)) {
+    errno = 0;
+    l = strtoul (name + 6, &end, 10);
+    if (errno != 0 || *end != 0 || l > G_MAXINT)
+      return NULL;
+    i = l - 16384;
+    for (walk = player->priv->roots; walk; walk = walk->next) {
+      SwfdecMovie *cur = walk->data;
+      if (cur->depth < i)
+	continue;
+      if (cur->depth == i)
+	return cur;
+      break;
+    }
+  }
 
   for (walk = movie->list; walk; walk = walk->next) {
     SwfdecMovie *cur = walk->data;
@@ -1076,7 +1045,7 @@ swfdec_movie_get_variable (SwfdecAsObject *object, SwfdecAsObject *orig,
     return TRUE;
 
   /* FIXME: check that this is correct */
-  if (swfdec_gc_object_get_context (object)->version > 5 && variable == SWFDEC_AS_STR__global) {
+  if (object->context->version > 5 && variable == SWFDEC_AS_STR__global) {
     SWFDEC_AS_VALUE_SET_OBJECT (val, SWFDEC_AS_OBJECT (movie->resource->sandbox));
     *flags = 0;
     return TRUE;
@@ -1157,7 +1126,7 @@ swfdec_movie_call_variable_listeners (SwfdecMovie *movie, const char *name,
     SwfdecMovieVariableListener *listener = iter->data;
 
     if (listener->name != name &&
-	(swfdec_gc_object_get_context (movie)->version >= 7 ||
+	(SWFDEC_AS_OBJECT (movie)->context->version >= 7 ||
 	 !swfdec_str_case_equal (listener->name, name)))
       continue;
 
@@ -1224,7 +1193,7 @@ typedef struct {
 
 static void
 swfdec_movie_do_render (SwfdecMovie *movie, cairo_t *cr,
-    const SwfdecColorTransform *ctrans)
+    const SwfdecColorTransform *ctrans, const SwfdecRect *inval)
 {
   static const cairo_matrix_t ident = { 1, 0, 0, 1, 0, 0};
   GList *g;
@@ -1232,51 +1201,39 @@ swfdec_movie_do_render (SwfdecMovie *movie, cairo_t *cr,
   GSList *clips = NULL;
   ClipEntry *clip = NULL;
 
-  if (movie->draws || movie->image) {
-    SwfdecRect inval;
+  /* exeute the movie's drawing commands */
+  for (walk = movie->draws; walk; walk = walk->next) {
+    SwfdecDraw *draw = walk->data;
 
-    cairo_clip_extents (cr, &inval.x0, &inval.y0, &inval.x1, &inval.y1);
+    if (!swfdec_rect_intersect (NULL, &draw->extents, inval))
+      continue;
+    
+    swfdec_draw_paint (draw, cr, ctrans);
+  }
 
-    /* exeute the movie's drawing commands */
-    for (walk = movie->draws; walk; walk = walk->next) {
-      SwfdecDraw *draw = walk->data;
-
-      if (!swfdec_rect_intersect (NULL, &draw->extents, &inval))
-	continue;
-      
-      swfdec_draw_paint (draw, cr, ctrans);
-    }
-
-    /* if the movie loaded an image, draw it here now */
-    /* FIXME: add check to only draw if inside clip extents */
-    if (movie->image) {
-      SwfdecRenderer *renderer = swfdec_renderer_get (cr);
-      cairo_surface_t *surface;
-      cairo_pattern_t *pattern;
-      double alpha = 1.0;
-      
-      if (swfdec_color_transform_is_mask (ctrans)) {
-	surface = NULL;
-      } else if (swfdec_color_transform_is_alpha (ctrans)) {
-	surface = swfdec_image_create_surface (movie->image, renderer);
-	alpha = ctrans->aa / 256.0;
-      } else {
-	surface = swfdec_image_create_surface_transformed (movie->image,
-	  renderer, ctrans);
-      }
-      if (surface) {
-	static const cairo_matrix_t matrix = { 1.0 / SWFDEC_TWIPS_SCALE_FACTOR, 0, 0, 1.0 / SWFDEC_TWIPS_SCALE_FACTOR, 0, 0 };
-	pattern = cairo_pattern_create_for_surface (surface);
-	SWFDEC_LOG ("rendering loaded image");
-	cairo_pattern_set_matrix (pattern, &matrix);
-      } else {
-	pattern = cairo_pattern_create_rgb (1.0, 0.0, 0.0);
-      }
-      cairo_set_source (cr, pattern);
-      cairo_paint_with_alpha (cr, alpha);
-      cairo_pattern_destroy (pattern);
-      cairo_surface_destroy (surface);
+  /* if the movie loaded an image, draw it here now */
+  if (movie->image) {
+    SwfdecRenderer *renderer = swfdec_renderer_get (cr);
+    cairo_surface_t *surface;
+    cairo_pattern_t *pattern;
+    
+    if (swfdec_color_transform_is_mask (ctrans))
+      surface = NULL;
+    else
+      surface = swfdec_image_create_surface_transformed (movie->image,
+	renderer, ctrans);
+    if (surface) {
+      static const cairo_matrix_t matrix = { 1.0 / SWFDEC_TWIPS_SCALE_FACTOR, 0, 0, 1.0 / SWFDEC_TWIPS_SCALE_FACTOR, 0, 0 };
+      pattern = cairo_pattern_create_for_surface (surface);
+      SWFDEC_LOG ("rendering loaded image");
+      cairo_pattern_set_matrix (pattern, &matrix);
+    } else {
+      pattern = cairo_pattern_create_rgb (1.0, 0.0, 0.0);
     }
+    cairo_set_source (cr, pattern);
+    cairo_paint (cr);
+    cairo_pattern_destroy (pattern);
+    cairo_surface_destroy (surface);
   }
 
   /* draw the children movies */
@@ -1286,7 +1243,7 @@ swfdec_movie_do_render (SwfdecMovie *movie, cairo_t *cr,
     while (clip && clip->depth < child->depth) {
       cairo_pattern_t *mask;
       SWFDEC_INFO ("unsetting clip depth %d for depth %d", clip->depth, child->depth);
-      mask = swfdec_movie_mask (cr, clip->movie, &ident);
+      mask = swfdec_movie_mask (cr, clip->movie, &ident, inval);
       cairo_pop_group_to_source (cr);
       cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
       cairo_mask (cr, mask);
@@ -1308,13 +1265,12 @@ swfdec_movie_do_render (SwfdecMovie *movie, cairo_t *cr,
     }
 
     SWFDEC_LOG ("rendering %p with depth %d", child, child->depth);
-    if (child->visible)
-      swfdec_movie_render (child, cr, ctrans);
+    swfdec_movie_render (child, cr, ctrans, inval);
   }
   while (clip) {
     cairo_pattern_t *mask;
     SWFDEC_INFO ("unsetting clip depth %d", clip->depth);
-    mask = swfdec_movie_mask (cr, clip->movie, &ident);
+    mask = swfdec_movie_mask (cr, clip->movie, &ident, inval);
     cairo_pop_group_to_source (cr);
     cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
     cairo_mask (cr, mask);
@@ -1341,44 +1297,24 @@ swfdec_movie_do_invalidate (SwfdecMovie *movie, const cairo_matrix_t *matrix, gb
   }
   swfdec_rect_union (&rect, &rect, &movie->draw_extents);
   swfdec_rect_transform (&rect, &rect, matrix);
-  swfdec_player_invalidate (SWFDEC_PLAYER (swfdec_gc_object_get_context (movie)), &rect);
+  swfdec_player_invalidate (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context), &rect);
 
   for (walk = movie->list; walk; walk = walk->next) {
     swfdec_movie_invalidate (walk->data, matrix, last);
   }
 }
 
-static GObject *
-swfdec_movie_constructor (GType type, guint n_construct_properties,
-    GObjectConstructParam *construct_properties)
-{
-  GObject *object;
-  SwfdecPlayerPrivate *priv;
-
-  object = G_OBJECT_CLASS (swfdec_movie_parent_class)->constructor (type, 
-      n_construct_properties, construct_properties);
-
-  priv = SWFDEC_PLAYER (swfdec_gc_object_get_context (object))->priv;
-  /* the movie is created invalid */
-  priv->invalid_pending = g_slist_prepend (priv->invalid_pending, object);
-
-  return object;
-}
-
 static void
 swfdec_movie_class_init (SwfdecMovieClass * movie_class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (movie_class);
-  SwfdecGcObjectClass *gc_class = SWFDEC_GC_OBJECT_CLASS (movie_class);
   SwfdecAsObjectClass *asobject_class = SWFDEC_AS_OBJECT_CLASS (movie_class);
 
-  object_class->constructor = swfdec_movie_constructor;
   object_class->dispose = swfdec_movie_dispose;
   object_class->get_property = swfdec_movie_get_property;
   object_class->set_property = swfdec_movie_set_property;
 
-  gc_class->mark = swfdec_movie_mark;
-
+  asobject_class->mark = swfdec_movie_mark;
   asobject_class->get = swfdec_movie_get_variable;
   asobject_class->set = swfdec_movie_set_variable;
   asobject_class->foreach = swfdec_movie_foreach_variable;
@@ -1391,18 +1327,6 @@ swfdec_movie_class_init (SwfdecMovieClass * movie_class)
   g_object_class_install_property (object_class, PROP_DEPTH,
       g_param_spec_int ("depth", "depth", "z order inside the parent",
 	  G_MININT, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-  g_object_class_install_property (object_class, PROP_GRAPHIC,
-      g_param_spec_object ("graphic", "graphic", "graphic represented by this movie",
-	  SWFDEC_TYPE_GRAPHIC, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-  g_object_class_install_property (object_class, PROP_NAME,
-      g_param_spec_string ("name", "name", "the name given to this movie (can be empty)",
-	  NULL, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-  g_object_class_install_property (object_class, PROP_PARENT,
-      g_param_spec_object ("parent", "parent", "parent movie containing this movie",
-	  SWFDEC_TYPE_MOVIE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-  g_object_class_install_property (object_class, PROP_RESOURCE,
-      g_param_spec_object ("resource", "resource", "the resource that spawned this movie",
-	  SWFDEC_TYPE_RESOURCE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
   movie_class->render = swfdec_movie_do_render;
   movie_class->invalidate = swfdec_movie_do_invalidate;
@@ -1436,12 +1360,28 @@ swfdec_movie_set_depth (SwfdecMovie *movie, int depth)
   if (movie->parent) {
     movie->parent->list = g_list_sort (movie->parent->list, swfdec_movie_compare_depths);
   } else {
-    SwfdecPlayerPrivate *player = SWFDEC_PLAYER (swfdec_gc_object_get_context (movie))->priv;
+    SwfdecPlayerPrivate *player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context)->priv;
     player->roots = g_list_sort (player->roots, swfdec_movie_compare_depths);
   }
   g_object_notify (G_OBJECT (movie), "depth");
 }
 
+static void
+swfdec_movie_set_version (SwfdecMovie *movie)
+{
+  SwfdecAsObject *o;
+  SwfdecAsContext *cx;
+  SwfdecAsValue val;
+
+  if (movie->parent != NULL)
+    return;
+
+  o = SWFDEC_AS_OBJECT (movie);
+  cx = o->context;
+  SWFDEC_AS_VALUE_SET_STRING (&val, swfdec_as_context_get_string (cx, SWFDEC_PLAYER (cx)->priv->system->version));
+  swfdec_as_object_set_variable (o, SWFDEC_AS_STR__version, &val);
+}
+
 /**
  * swfdec_movie_new:
  * @player: a #SwfdecPlayer
@@ -1467,7 +1407,7 @@ swfdec_movie_new (SwfdecPlayer *player, int depth, SwfdecMovie *parent, SwfdecRe
     SwfdecGraphic *graphic, const char *name)
 {
   SwfdecMovie *movie;
-  GType type;
+  gsize size;
 
   g_return_val_if_fail (SWFDEC_IS_PLAYER (player), NULL);
   g_return_val_if_fail (parent == NULL || SWFDEC_IS_MOVIE (parent), NULL);
@@ -1476,15 +1416,63 @@ swfdec_movie_new (SwfdecPlayer *player, int depth, SwfdecMovie *parent, SwfdecRe
 
   /* create the right movie */
   if (graphic == NULL) {
-    type = SWFDEC_TYPE_SPRITE_MOVIE;
+    movie = g_object_new (SWFDEC_TYPE_SPRITE_MOVIE, "depth", depth, NULL);
+    size = sizeof (SwfdecSpriteMovie);
   } else {
     SwfdecGraphicClass *klass = SWFDEC_GRAPHIC_GET_CLASS (graphic);
-    g_return_val_if_fail (g_type_is_a (klass->movie_type, SWFDEC_TYPE_MOVIE), NULL);
-    type = klass->movie_type;
+    g_return_val_if_fail (klass->create_movie != NULL, NULL);
+    movie = klass->create_movie (graphic, &size);
+    movie->graphic = g_object_ref (graphic);
+    movie->depth = depth;
   }
-  movie = g_object_new (type, "context", player, "depth", depth, 
-      "parent", parent, "name", name, "resource", resource,
-      "graphic", graphic, NULL);
+  /* register it to the VM */
+  /* FIXME: It'd be nice if we'd not overuse memory here when calling this function from a script */
+  if (!swfdec_as_context_use_mem (SWFDEC_AS_CONTEXT (player), size)) {
+    size = 0;
+  }
+  g_object_ref (movie);
+  /* set essential properties */
+  movie->parent = parent;
+  movie->resource = resource;
+  if (parent) {
+    parent->list = g_list_insert_sorted (parent->list, movie, swfdec_movie_compare_depths);
+    SWFDEC_DEBUG ("inserting %s %s (depth %d) into %s %p", G_OBJECT_TYPE_NAME (movie), movie->name,
+	movie->depth,  G_OBJECT_TYPE_NAME (parent), parent);
+    /* invalidate the parent, so it gets visible */
+    swfdec_movie_queue_update (parent, SWFDEC_MOVIE_INVALID_CHILDREN);
+  } else {
+    player->priv->roots = g_list_insert_sorted (player->priv->roots, movie, swfdec_movie_compare_depths);
+  }
+  /* set its name */
+  if (name) {
+    movie->original_name = name;
+    movie->name = name;
+  } else {
+    movie->original_name = SWFDEC_AS_STR_EMPTY;
+    if (SWFDEC_IS_SPRITE_MOVIE (movie) || SWFDEC_IS_BUTTON_MOVIE (movie)) {
+      movie->name = swfdec_as_context_give_string (SWFDEC_AS_CONTEXT (player), 
+	  g_strdup_printf ("instance%u", ++player->priv->unnamed_count));
+    } else {
+      movie->name = SWFDEC_AS_STR_EMPTY;
+    }
+  }
+  /* add the movie to the global movies list */
+  /* NB: adding to the movies list happens before setting the parent.
+   * Setting the parent does a gotoAndPlay(0) for Sprites which can cause
+   * new movies to be created (and added to this list)
+   */
+  if (SWFDEC_IS_ACTOR (movie))
+    player->priv->actors = g_list_prepend (player->priv->actors, movie);
+  /* only add the movie here, because it needs to be setup for the debugger */
+  swfdec_as_object_add (SWFDEC_AS_OBJECT (movie), SWFDEC_AS_CONTEXT (player), size);
+  swfdec_movie_set_version (movie);
+  /* only setup here, the resource assumes it can access the player via the movie */
+  if (resource->movie == NULL) {
+    g_assert (SWFDEC_IS_SPRITE_MOVIE (movie));
+    resource->movie = SWFDEC_SPRITE_MOVIE (movie);
+  }
+  /* the movie is invalid already */
+  player->priv->invalid_pending = g_slist_prepend (player->priv->invalid_pending, movie);
 
   return movie;
 }
@@ -1518,7 +1506,7 @@ swfdec_movie_set_static_properties (SwfdecMovie *movie, const cairo_matrix_t *tr
   }
   if (ctrans) {
     swfdec_movie_invalidate_last (movie);
-    movie->color_transform = *ctrans;
+    movie->original_ctrans = *ctrans;
   }
   if (ratio >= 0 && (guint) ratio != movie->original_ratio) {
     SwfdecMovieClass *klass;
@@ -1580,14 +1568,14 @@ swfdec_movie_duplicate (SwfdecMovie *movie, const char *name, int depth)
     SWFDEC_LOG ("depth %d already occupied while duplicating, removing old movie", depth);
     swfdec_movie_remove (copy);
   }
-  copy = swfdec_movie_new (SWFDEC_PLAYER (swfdec_gc_object_get_context (movie)), depth, 
+  copy = swfdec_movie_new (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context), depth, 
       parent, movie->resource, movie->graphic, name);
   if (copy == NULL)
     return NULL;
   swfdec_movie_set_static_properties (copy, &movie->original_transform,
-      &movie->color_transform, movie->original_ratio, movie->clip_depth, 
+      &movie->original_ctrans, movie->original_ratio, movie->clip_depth, 
       movie->blend_mode, SWFDEC_IS_ACTOR (movie) ? SWFDEC_ACTOR (movie)->events : NULL);
-  sandbox = SWFDEC_SANDBOX (swfdec_gc_object_get_context (movie)->global);
+  sandbox = SWFDEC_SANDBOX (SWFDEC_AS_OBJECT (movie)->context->global);
   swfdec_sandbox_unuse (sandbox);
   if (SWFDEC_IS_SPRITE_MOVIE (copy)) {
     SwfdecActor *actor = SWFDEC_ACTOR (copy);
