@@ -129,6 +129,7 @@ struct _SwfdecMovie {
 
   /* static properties (set by PlaceObject tags) */
   const char *		original_name;		/* the original name - GC'd and static */
+  SwfdecColorTransform	original_ctrans;	/* color transform set on this movie */
   cairo_matrix_t	original_transform;	/* initial transform used */
   guint			original_ratio;		/* ratio used in this movie */
   int			clip_depth;		/* up to which movie this movie clips */
@@ -194,7 +195,8 @@ struct _SwfdecMovieClass {
 						 SwfdecRect *   	extents);
   void			(* render)		(SwfdecMovie *		movie, 
 						 cairo_t *		cr,
-						 const SwfdecColorTransform *trans);
+						 const SwfdecColorTransform *trans,
+						 const SwfdecRect *	inval);
   void			(* invalidate)		(SwfdecMovie *		movie,
 						 const cairo_matrix_t *	movie_to_global,
 						 gboolean		new_contents);
@@ -281,8 +283,8 @@ char *		swfdec_movie_get_path		(SwfdecMovie *		movie,
 						 gboolean		dot);
 void		swfdec_movie_render		(SwfdecMovie *		movie,
 						 cairo_t *		cr, 
-						 const SwfdecColorTransform *trans);
-gboolean	swfdec_movie_is_scriptable	(SwfdecMovie *		movie);
+						 const SwfdecColorTransform *trans,
+						 const SwfdecRect *	inval);
 SwfdecMovie *	swfdec_movie_resolve		(SwfdecMovie *		movie);
 guint		swfdec_movie_get_version	(SwfdecMovie *		movie);
 
