@@ -152,8 +152,8 @@ swfdec_color_pattern_get_pattern (SwfdecPattern *pat, SwfdecRenderer *renderer,
 
   color = swfdec_color_apply_transform (color, trans);
   return cairo_pattern_create_rgba ( 
-      SWFDEC_COLOR_RED (color) / 255.0, SWFDEC_COLOR_GREEN (color) / 255.0,
-      SWFDEC_COLOR_BLUE (color) / 255.0, SWFDEC_COLOR_ALPHA (color) / 255.0);
+      SWFDEC_COLOR_R (color) / 255.0, SWFDEC_COLOR_G (color) / 255.0,
+      SWFDEC_COLOR_B (color) / 255.0, SWFDEC_COLOR_A (color) / 255.0);
 }
 
 static void
@@ -211,32 +211,6 @@ swfdec_image_pattern_morph (SwfdecDraw *dest, SwfdecDraw *source, guint ratio)
   SWFDEC_DRAW_CLASS (swfdec_image_pattern_parent_class)->morph (dest, source, ratio);
 }
 
-static void
-swfdec_image_pattern_paint (SwfdecDraw *draw, cairo_t *cr, const SwfdecColorTransform *trans)
-{
-  cairo_pattern_t *pattern;
-
-  if (swfdec_color_transform_is_alpha (trans)) {
-    SwfdecColorTransform identity;
-    swfdec_color_transform_init_identity (&identity);
-    pattern = swfdec_pattern_get_pattern (SWFDEC_PATTERN (draw),
-	swfdec_renderer_get (cr), &identity);
-    if (pattern == NULL)
-      return;
-    cairo_save (cr);
-    cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
-    cairo_set_fill_rule (cr, CAIRO_FILL_RULE_EVEN_ODD);
-    cairo_append_path (cr, &draw->path);
-    cairo_clip (cr);
-    cairo_set_source (cr, pattern);
-    cairo_pattern_destroy (pattern);
-    cairo_paint_with_alpha (cr, trans->aa / 256.0);
-    cairo_restore (cr);
-  } else {
-    SWFDEC_DRAW_CLASS (swfdec_image_pattern_parent_class)->paint (draw, cr, trans);
-  }
-}
-
 static cairo_pattern_t *
 swfdec_image_pattern_get_pattern (SwfdecPattern *pat, SwfdecRenderer *renderer,
     const SwfdecColorTransform *trans)
@@ -262,7 +236,6 @@ static void
 swfdec_image_pattern_class_init (SwfdecImagePatternClass *klass)
 {
   SWFDEC_DRAW_CLASS (klass)->morph = swfdec_image_pattern_morph;
-  SWFDEC_DRAW_CLASS (klass)->paint = swfdec_image_pattern_paint;
 
   SWFDEC_PATTERN_CLASS (klass)->get_pattern = swfdec_image_pattern_get_pattern;
 }
