@@ -158,10 +158,18 @@ void		swfdec_player_set_allow_fullscreen
 						 gboolean		allow);
 					 
 void		swfdec_player_render		(SwfdecPlayer *		player,
-						 cairo_t *		cr);
+						 cairo_t *		cr,
+						 double			x,
+						 double			y,
+						 double			width,
+						 double			height);
 void		swfdec_player_render_with_renderer (SwfdecPlayer *	player,
 						 cairo_t *		cr,
-						 SwfdecRenderer *	renderer);
+						 SwfdecRenderer *	renderer,
+						 double			x,
+						 double			y,
+						 double			width,
+						 double			height);
 gulong		swfdec_player_advance		(SwfdecPlayer *		player,
 						 gulong			msecs);
 gboolean	swfdec_player_mouse_move	(SwfdecPlayer *		player, 
