@@ -57,19 +57,19 @@ swfdec_resource_is_root (SwfdecResource *resource)
   g_return_val_if_fail (SWFDEC_IS_RESOURCE (resource), FALSE);
 
   return
-    resource->movie == SWFDEC_PLAYER (swfdec_gc_object_get_context (resource))->priv->roots->data;
+    resource->movie == SWFDEC_PLAYER (SWFDEC_AS_OBJECT (resource)->context)->priv->roots->data;
 }
 
 static SwfdecPlayer *
 swfdec_resource_stream_target_get_player (SwfdecStreamTarget *target)
 {
-  return SWFDEC_PLAYER (swfdec_gc_object_get_context (target));
+  return SWFDEC_PLAYER (SWFDEC_AS_OBJECT (target)->context);
 }
 
 static void
 swfdec_resource_stream_target_image (SwfdecResource *instance)
 {
-  SwfdecPlayer *player = SWFDEC_PLAYER (swfdec_gc_object_get_context (instance));
+  SwfdecPlayer *player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (instance)->context);
   SwfdecSpriteMovie *movie = instance->movie;
 
   if (movie->sprite != NULL)
@@ -118,24 +118,32 @@ swfdec_resource_emit_signal (SwfdecResource *resource, const char *name, gboolea
 
   if (resource->clip_loader == NULL)
     return;
-  cx = swfdec_gc_object_get_context (resource->clip_loader);
+  cx = SWFDEC_AS_OBJECT (resource->clip_loader)->context;
+  /* This feels wrong. Why do we resolve here by real name? */
+  if (resource->target) {
+    SwfdecMovie *parent = swfdec_movie_resolve (resource->target->parent);
+    if (parent)
+      movie = swfdec_movie_get_by_name (parent, resource->target->name, FALSE);
+    else
+      movie = NULL;
+  } else {
+    movie = NULL;
+  }
+  if (movie == NULL && resource->movie != NULL) {
+    SWFDEC_DEBUG ("no movie, not emitting signal");
+    return;
+  }
+  if (name == SWFDEC_AS_STR_onLoadInit &&
+      movie != SWFDEC_MOVIE (resource->movie)) {
+    SWFDEC_INFO ("not emitting onLoadInit - the movie is different");
+    return;
+  }
 
   SWFDEC_AS_VALUE_SET_STRING (&vals[0], name);
-  if (resource->movie) {
-    movie = swfdec_movie_resolve (SWFDEC_MOVIE (resource->movie));
-    if (movie == NULL) {
-      SWFDEC_DEBUG ("no movie, not emitting signal");
-      return;
-    }
-    if (name == SWFDEC_AS_STR_onLoadInit &&
-	movie != SWFDEC_MOVIE (resource->movie)) {
-      SWFDEC_INFO ("not emitting onLoadInit - the movie is different");
-      return;
-    }
+  if (movie) {
     SWFDEC_AS_VALUE_SET_OBJECT (&vals[1], SWFDEC_AS_OBJECT (movie));
   } else {
     SWFDEC_AS_VALUE_SET_UNDEFINED (&vals[1]);
-    movie = NULL;
   }
   if (progress) {
     SwfdecResource *res;
@@ -180,7 +188,7 @@ swfdec_resource_replace_movie (SwfdecSpriteMovie *movie, SwfdecResource *resourc
   SwfdecMovie *mov = SWFDEC_MOVIE (movie);
   SwfdecMovie *copy;
   
-  copy = swfdec_movie_new (SWFDEC_PLAYER (swfdec_gc_object_get_context (movie)), 
+  copy = swfdec_movie_new (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (movie)->context), 
       mov->depth, mov->parent, resource, NULL, mov->name);
   if (copy == NULL)
     return FALSE;
@@ -195,7 +203,7 @@ swfdec_resource_replace_movie (SwfdecSpriteMovie *movie, SwfdecResource *resourc
   swfdec_movie_end_update_matrix (copy);
   /* FIXME: are events copied? If so, wouldn't that be a security issue? */
   swfdec_movie_set_static_properties (copy, &mov->original_transform,
-      &mov->color_transform, mov->original_ratio, mov->clip_depth, 
+      &mov->original_ctrans, mov->original_ratio, mov->clip_depth, 
       mov->blend_mode, NULL);
   swfdec_movie_remove (mov);
   return SWFDEC_SPRITE_MOVIE (copy);
@@ -246,7 +254,7 @@ swfdec_resource_stream_target_parse (SwfdecStreamTarget *target, SwfdecStream *s
       glong total;
       resource->decoder = dec;
       g_signal_connect_swapped (dec, "missing-plugin", 
-	  G_CALLBACK (swfdec_player_add_missing_plugin), swfdec_gc_object_get_context (resource));
+	  G_CALLBACK (swfdec_player_add_missing_plugin), SWFDEC_AS_OBJECT (resource)->context);
       total = swfdec_loader_get_size (loader);
       if (total >= 0)
 	dec->bytes_total = total;
@@ -282,7 +290,7 @@ swfdec_resource_stream_target_parse (SwfdecStreamTarget *target, SwfdecStream *s
       if (SWFDEC_IS_SWF_DECODER (dec))
 	resource->version = SWFDEC_SWF_DECODER (dec)->version;
       if (swfdec_resource_is_root (resource)) {
-	swfdec_player_initialize (SWFDEC_PLAYER (swfdec_gc_object_get_context (resource)),
+	swfdec_player_initialize (SWFDEC_PLAYER (SWFDEC_AS_OBJECT (resource)->context),
 	    dec->rate, dec->width, dec->height);
       }
     }
@@ -301,7 +309,7 @@ swfdec_resource_abort_if_not_initialized (SwfdecResource *resource)
   if (resource->sandbox)
     return FALSE;
 
-  swfdec_as_context_abort (swfdec_gc_object_get_context (resource),
+  swfdec_as_context_abort (SWFDEC_AS_OBJECT (resource)->context,
       "This is not a Flash file");
   return TRUE;
 }
@@ -366,20 +374,20 @@ swfdec_resource_stream_target_init (SwfdecStreamTargetInterface *iface)
 }
 
 static void
-swfdec_resource_mark (SwfdecGcObject *object)
+swfdec_resource_mark (SwfdecAsObject *object)
 {
   SwfdecResource *resource = SWFDEC_RESOURCE (object);
 
   if (resource->clip_loader) {
-    swfdec_gc_object_mark (resource->clip_loader);
-    swfdec_gc_object_mark (resource->clip_loader_sandbox);
+    swfdec_as_object_mark (SWFDEC_AS_OBJECT (resource->clip_loader));
+    swfdec_as_object_mark (SWFDEC_AS_OBJECT (resource->clip_loader_sandbox));
   }
   if (resource->sandbox)
-    swfdec_gc_object_mark (resource->sandbox);
+    swfdec_as_object_mark (SWFDEC_AS_OBJECT (resource->sandbox));
   if (resource->target)
-    swfdec_gc_object_mark (resource->target);
+    swfdec_as_object_mark (SWFDEC_AS_OBJECT (resource->target));
 
-  SWFDEC_GC_OBJECT_CLASS (swfdec_resource_parent_class)->mark (object);
+  SWFDEC_AS_OBJECT_CLASS (swfdec_resource_parent_class)->mark (object);
 }
 
 static void
@@ -394,7 +402,7 @@ swfdec_resource_dispose (GObject *object)
   }
   if (resource->decoder) {
     g_signal_handlers_disconnect_by_func (resource->decoder,
-	  swfdec_player_add_missing_plugin, swfdec_gc_object_get_context (resource));
+	  swfdec_player_add_missing_plugin, SWFDEC_AS_OBJECT (resource)->context);
     g_object_unref (resource->decoder);
     resource->decoder = NULL;
   }
@@ -409,11 +417,11 @@ static void
 swfdec_resource_class_init (SwfdecResourceClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  SwfdecGcObjectClass *gc_class = SWFDEC_GC_OBJECT_CLASS (klass);
+  SwfdecAsObjectClass *asobject_class = SWFDEC_AS_OBJECT_CLASS (klass);
 
   object_class->dispose = swfdec_resource_dispose;
 
-  gc_class->mark = swfdec_resource_mark;
+  asobject_class->mark = swfdec_resource_mark;
 }
 
 static void
@@ -441,11 +449,16 @@ SwfdecResource *
 swfdec_resource_new (SwfdecPlayer *player, SwfdecLoader *loader, const char *variables)
 {
   SwfdecResource *resource;
+  guint size;
 
   g_return_val_if_fail (SWFDEC_IS_PLAYER (player), NULL);
   g_return_val_if_fail (SWFDEC_IS_LOADER (loader), NULL);
 
-  resource = g_object_new (SWFDEC_TYPE_RESOURCE, "context", player, NULL);
+  size = sizeof (SwfdecResource);
+  if (!swfdec_as_context_use_mem (SWFDEC_AS_CONTEXT (player), size))
+    size = 0;
+  resource = g_object_new (SWFDEC_TYPE_RESOURCE, NULL);
+  swfdec_as_object_add (SWFDEC_AS_OBJECT (resource), SWFDEC_AS_CONTEXT (player), size);
   resource->version = 8;
   resource->variables = g_strdup (variables);
   swfdec_resource_set_loader (resource, loader);
@@ -511,11 +524,11 @@ swfdec_resource_load_mark (gpointer loadp, gpointer playerp)
 {
   SwfdecResourceLoad *load = loadp;
 
-  swfdec_gc_object_mark (load->sandbox);
+  swfdec_as_object_mark (SWFDEC_AS_OBJECT (load->sandbox));
   if (load->loader)
-    swfdec_gc_object_mark (load->loader);
+    swfdec_as_object_mark (SWFDEC_AS_OBJECT (load->loader));
   if (load->target_movie)
-    swfdec_gc_object_mark (load->target_movie);
+    swfdec_as_object_mark (SWFDEC_AS_OBJECT (load->target_movie));
 }
 
 static gboolean
@@ -526,7 +539,7 @@ swfdec_resource_create_movie (SwfdecResource *resource, SwfdecResourceLoad *load
 
   if (resource->movie)
     return TRUE;
-  player = SWFDEC_PLAYER (swfdec_gc_object_get_context (resource));
+  player = SWFDEC_PLAYER (SWFDEC_AS_OBJECT (resource)->context);
   if (load->target_movie) {
     movie = (SwfdecSpriteMovie *) swfdec_movie_resolve (SWFDEC_MOVIE (load->target_movie));
     if (SWFDEC_IS_SPRITE_MOVIE (movie))
@@ -534,15 +547,11 @@ swfdec_resource_create_movie (SwfdecResource *resource, SwfdecResourceLoad *load
     else
       movie = NULL;
   } else {
-    int level = swfdec_player_get_level (player, load->target_string, 7);
-    if (level >= 0) {
-      movie = swfdec_player_get_movie_at_level (player, level);
-      if (movie)
-	swfdec_movie_remove (SWFDEC_MOVIE (movie));
+    int level = swfdec_player_get_level (player, load->target_string);
+    if (level >= 0)
       movie = swfdec_player_create_movie_at_level (player, resource, level);
-    } else {
+    else
       movie = NULL;
-    }
   }
   if (movie == NULL) {
     SWFDEC_WARNING ("target does not reference a movie, not loading %s", load->url);
@@ -560,21 +569,24 @@ swfdec_resource_do_load (SwfdecPlayer *player, gboolean allowed, gpointer loadp)
   SwfdecResource *resource;
   SwfdecLoader *loader;
   
-  resource = g_object_new (SWFDEC_TYPE_RESOURCE, "context", player, NULL);
+  if (!swfdec_as_context_use_mem (SWFDEC_AS_CONTEXT (player), sizeof (SwfdecResource))) {
+    swfdec_player_unroot (player, load);
+    return;
+  }
+  resource = g_object_new (SWFDEC_TYPE_RESOURCE, NULL);
+  swfdec_as_object_add (SWFDEC_AS_OBJECT (resource), SWFDEC_AS_CONTEXT (player), sizeof (SwfdecResource));
   resource->version = 8;
   if (load->loader) {
     resource->clip_loader = load->loader;
     resource->clip_loader_sandbox = load->sandbox;
   }
+  resource->target = SWFDEC_MOVIE (load->target_movie);
   resource->sandbox = load->sandbox;
   if (!allowed) {
     SWFDEC_WARNING ("SECURITY: no access to %s from %s",
 	load->url, swfdec_url_get_url (load->sandbox->url));
     /* FIXME: is replacing correct? */
-    if (load->target_movie) {
-      resource->movie = SWFDEC_SPRITE_MOVIE (swfdec_movie_resolve (SWFDEC_MOVIE (load->target_movie)));
-      swfdec_resource_emit_error (resource, SWFDEC_AS_STR_IllegalRequest);
-    }
+    swfdec_resource_emit_error (resource, SWFDEC_AS_STR_IllegalRequest);
     swfdec_player_unroot (player, load);
     return;
   }
@@ -633,7 +645,7 @@ swfdec_resource_load_request (gpointer loadp, gpointer playerp)
   }
 
   /* LAUNCH command (aka getURL) */
-  if (load->target_string && swfdec_player_get_level (player, load->target_string, 7) < 0) {
+  if (load->target_string && swfdec_player_get_level (player, load->target_string) < 0) {
     swfdec_player_launch (player, load->url, load->target_string, load->buffer);
     swfdec_player_unroot (player, load);
     return;
@@ -687,38 +699,17 @@ swfdec_resource_load_movie (SwfdecPlayer *player, const SwfdecAsValue *target,
     }
   }
 
-  if (loader) {
-    if (SWFDEC_AS_VALUE_IS_NUMBER (target)) {
-      int i = swfdec_as_double_to_integer (SWFDEC_AS_VALUE_GET_NUMBER (target));
-      if (i < 0)
-	return FALSE;
-      s = swfdec_as_context_give_string (swfdec_gc_object_get_context (loader),
-	  g_strdup_printf ("_level%d", i));
-      swfdec_resource_load_internal (player, NULL, s, url, buffer, loader);
-      return TRUE;
-    } else if (SWFDEC_AS_VALUE_IS_STRING (target) ||
-	(SWFDEC_AS_VALUE_IS_OBJECT (target) && SWFDEC_IS_MOVIE (SWFDEC_AS_VALUE_GET_OBJECT (target)))) {
-      s = swfdec_as_value_to_string (SWFDEC_AS_CONTEXT (player), target);
-    } else {
-      SWFDEC_WARNING ("target does not reference a movie, not loading %s", url);
-      return FALSE;
-    }
-  } else {
-    s = swfdec_as_value_to_string (SWFDEC_AS_CONTEXT (player), target);
-  }
-  if (swfdec_player_get_level (player, s, SWFDEC_AS_CONTEXT (player)->version) >= 0) {
-    /* lowercase the string, so we can do case insensitive level lookups later on */
-    char *tmp = g_ascii_strdown (s, -1);
-    swfdec_resource_load_internal (player, NULL, tmp, url, buffer, NULL);
-    g_free (tmp);
-    return TRUE;
-  }
+  s = swfdec_as_value_to_string (SWFDEC_AS_CONTEXT (player), target);
   movie = swfdec_player_get_movie_from_string (player, s);
   if (SWFDEC_IS_SPRITE_MOVIE (movie)) {
     swfdec_resource_load_internal (player, SWFDEC_SPRITE_MOVIE (movie),
 	NULL, url, buffer, loader);
     return TRUE;
   }
+  if (swfdec_player_get_level (player, s) >= 0) {
+    swfdec_resource_load_internal (player, NULL, s, url, buffer, NULL);
+    return TRUE;
+  }
   SWFDEC_WARNING ("%s does not reference a movie, not loading %s", s, url);
   return FALSE;
 }
