@@ -1,7 +1,7 @@
 /* Swfdec
  * Copyright (C) 2003-2006 David Schleef <ds@schleef.org>
  *		 2005-2006 Eric Anholt <eric@anholt.net>
- *		 2006-2008 Benjamin Otte <otte@gnome.org>
+ *		 2006-2007 Benjamin Otte <otte@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,8 +26,6 @@
 #include "swfdec_movie.h"
 #include "swfdec_as_internal.h"
 #include "swfdec_as_strings.h"
-#include "swfdec_bitmap_data.h"
-#include "swfdec_bitmap_movie.h"
 #include "swfdec_bits.h"
 #include "swfdec_debug.h"
 #include "swfdec_decoder.h"
@@ -36,7 +34,6 @@
 #include "swfdec_sprite.h"
 #include "swfdec_sprite_movie.h"
 #include "swfdec_swf_decoder.h"
-#include "swfdec_transform_as.h"
 #include "swfdec_resource.h"
 #include "swfdec_utils.h"
 #include "swfdec_as_internal.h"
@@ -151,12 +148,7 @@ void
 swfdec_sprite_movie_get_transform (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *rval)
 {
-  SwfdecMovie *movie;
-
-  SWFDEC_AS_CHECK (SWFDEC_TYPE_MOVIE, &movie, "");
-
-  SWFDEC_AS_VALUE_SET_OBJECT (rval,
-      SWFDEC_AS_OBJECT (swfdec_transform_as_new (cx, movie)));
+  SWFDEC_STUB ("MovieClip.transform (get)");
 }
 
 SWFDEC_AS_NATIVE (900, 420, swfdec_sprite_movie_set_transform)
@@ -323,11 +315,8 @@ swfdec_sprite_movie_getInstanceAtDepth (SwfdecAsContext *cx,
     return;
 
   movie = swfdec_movie_find (movie, depth);
-  if (movie != NULL) {
-    if (!swfdec_movie_is_scriptable (movie))
-      movie = movie->parent;
+  if (movie != NULL)
     SWFDEC_AS_VALUE_SET_OBJECT (rval, SWFDEC_AS_OBJECT (movie));
-  }
 }
 
 SWFDEC_AS_NATIVE (900, 24, swfdec_sprite_movie_getSWFVersion)
@@ -348,28 +337,7 @@ void
 swfdec_sprite_movie_attachBitmap (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *rval)
 {
-  SwfdecMovie *parent;
-  SwfdecBitmapData *bitmap;
-  const char *snapping;
-  gboolean smoothing;
-  int depth;
-  SwfdecMovie *movie;
-
-  SWFDEC_AS_CHECK (SWFDEC_TYPE_SPRITE_MOVIE, &parent, "oi|sb", 
-      &bitmap, &depth, &snapping, &smoothing);
-
-  if (!SWFDEC_IS_BITMAP_DATA (bitmap))
-    return;
-  if (swfdec_depth_classify (depth) == SWFDEC_DEPTH_CLASS_EMPTY)
-    return;
-
-  movie = swfdec_movie_find (parent, depth);
-  if (movie)
-    swfdec_movie_remove (movie);
-
-  swfdec_bitmap_movie_new (parent, bitmap, depth);
-  SWFDEC_LOG ("created new BitmapMovie to parent %s at depth %d", 
-      parent->name, depth);
+  SWFDEC_STUB ("MovieClip.attachBitmap");
 }
 
 SWFDEC_AS_NATIVE (900, 26, swfdec_sprite_movie_getRect)
@@ -480,7 +448,7 @@ swfdec_sprite_movie_do_goto (SwfdecSpriteMovie *movie, SwfdecAsValue *target)
       return;
     frame++;
   } else {
-    frame = swfdec_as_value_to_integer (swfdec_gc_object_get_context (movie), target);
+    frame = swfdec_as_value_to_integer (SWFDEC_AS_OBJECT (movie)->context, target);
   }
   /* FIXME: how to handle overflow? */
   frame = CLAMP (frame, 1, (int) movie->n_frames);
@@ -634,23 +602,23 @@ void
 swfdec_sprite_movie_swapDepths (SwfdecAsContext *cx, SwfdecAsObject *object,
     guint argc, SwfdecAsValue *argv, SwfdecAsValue *rval)
 {
-  SwfdecMovie *movie, *other;
-  SwfdecAsValue value;
+  SwfdecMovie *movie;
+  SwfdecMovie *other;
   int depth;
 
-  SWFDEC_AS_CHECK (SWFDEC_TYPE_MOVIE, (gpointer)&movie, "v", &value);
+  SWFDEC_AS_CHECK (SWFDEC_TYPE_MOVIE, (gpointer)&movie, "");
 
-  if (movie->parent == NULL)
-    SWFDEC_FIXME ("swapDepths on root movie, should do something weird");
+  if (argc < 1)
+    return;
 
-  if (SWFDEC_AS_VALUE_IS_OBJECT (&value)) {
-    other = (SwfdecMovie *) SWFDEC_AS_VALUE_GET_OBJECT (&value);
+  if (SWFDEC_AS_VALUE_IS_OBJECT (&argv[0])) {
+    other = (SwfdecMovie *) SWFDEC_AS_VALUE_GET_OBJECT (&argv[0]);
     if (!SWFDEC_IS_MOVIE (other) ||
 	other->parent != movie->parent)
       return;
     depth = other->depth;
   } else {
-    depth = swfdec_as_value_to_integer (cx, &value);
+    depth = swfdec_as_value_to_integer (cx, &argv[0]);
     if (movie->parent) {
       other = swfdec_movie_find (movie->parent, depth);
     } else {
@@ -658,13 +626,6 @@ swfdec_sprite_movie_swapDepths (SwfdecAsContext *cx, SwfdecAsObject *object,
       other = NULL;
     }
   }
-
-  // FIXME: one different than the reserved range, should the classify function
-  // be changed instead?
-  if (swfdec_depth_classify (depth) == SWFDEC_DEPTH_CLASS_EMPTY ||
-      depth >= 2130690045)
-    return;
-
   if (other)
     swfdec_movie_set_depth (other, movie->depth);
   swfdec_movie_set_depth (movie, depth);
@@ -685,18 +646,7 @@ swfdec_sprite_movie_createEmptyMovieClip (SwfdecAsContext *cx, SwfdecAsObject *o
   if (movie)
     swfdec_movie_remove (movie);
   movie = swfdec_movie_new (SWFDEC_PLAYER (cx), depth, parent, parent->resource, NULL, name);
-
-  if (SWFDEC_IS_SPRITE_MOVIE (movie)) {
-    SwfdecSandbox *sandbox = SWFDEC_SANDBOX (cx->global);
-    SwfdecActor *actor = SWFDEC_ACTOR (movie);
-    swfdec_sandbox_unuse (sandbox);
-    swfdec_movie_initialize (movie);
-    swfdec_actor_execute (actor, SWFDEC_EVENT_CONSTRUCT, 0);
-    swfdec_sandbox_use (sandbox);
-  } else {
-    swfdec_movie_initialize (movie);
-  }
-
+  swfdec_movie_initialize (movie);
   SWFDEC_AS_VALUE_SET_OBJECT (rval, SWFDEC_AS_OBJECT (movie));
 }
 
@@ -740,18 +690,8 @@ swfdec_sprite_movie_init_from_object (SwfdecMovie *movie,
   g_return_if_fail (initObject == NULL || SWFDEC_IS_AS_OBJECT (initObject));
 
   if (initObject != NULL) {
-    SwfdecAsContext *cx = swfdec_gc_object_get_context (movie);
-    if (cx->version <= 6) {
-      swfdec_as_object_foreach (initObject,
-	  swfdec_sprite_movie_foreach_copy_properties, SWFDEC_AS_OBJECT (movie));
-      swfdec_movie_initialize (movie);
-    } else {
-      swfdec_movie_initialize (movie);
-      swfdec_as_object_foreach (initObject,
-	  swfdec_sprite_movie_foreach_copy_properties, SWFDEC_AS_OBJECT (movie));
-    }
-  } else {
-    swfdec_movie_initialize (movie);
+    swfdec_as_object_foreach (initObject,
+	swfdec_sprite_movie_foreach_copy_properties, SWFDEC_AS_OBJECT (movie));
   }
 
   if (SWFDEC_IS_SPRITE_MOVIE (movie)) {
@@ -760,8 +700,11 @@ swfdec_sprite_movie_init_from_object (SwfdecMovie *movie,
     swfdec_actor_queue_script (actor, SWFDEC_EVENT_INITIALIZE);
     swfdec_actor_queue_script (actor, SWFDEC_EVENT_LOAD);
     swfdec_sandbox_unuse (sandbox);
+    swfdec_movie_initialize (movie);
     swfdec_actor_execute (actor, SWFDEC_EVENT_CONSTRUCT, 0);
     swfdec_sandbox_use (sandbox);
+  } else {
+    swfdec_movie_initialize (movie);
   }
 }
 
@@ -798,8 +741,7 @@ swfdec_sprite_movie_attachMovie (SwfdecAsContext *cx, SwfdecAsObject *object,
   ret = swfdec_movie_find (movie, depth);
   if (ret)
     swfdec_movie_remove (ret);
-  ret = swfdec_movie_new (SWFDEC_PLAYER (swfdec_gc_object_get_context (object)),
-      depth, movie, movie->resource, sprite, name);
+  ret = swfdec_movie_new (SWFDEC_PLAYER (object->context), depth, movie, movie->resource, sprite, name);
   SWFDEC_LOG ("attached %s (%u) as %s to depth %u", export, SWFDEC_CHARACTER (sprite)->id,
       ret->name, ret->depth);
   /* run init and construct */
@@ -872,6 +814,8 @@ swfdec_sprite_movie_getBounds (SwfdecAsContext *cx, SwfdecAsObject *object,
   SWFDEC_AS_CHECK (SWFDEC_TYPE_MOVIE, &movie, "");
 
   obj= swfdec_as_object_new_empty (cx);
+  if (obj== NULL)
+    return;
 
   swfdec_movie_update (movie);
   if (swfdec_rect_is_empty (&movie->extents)) {
@@ -964,8 +908,12 @@ swfdec_sprite_movie_init_context (SwfdecPlayer *player)
 
   movie = SWFDEC_AS_OBJECT (swfdec_as_object_add_function (context->global, 
       SWFDEC_AS_STR_MovieClip, 0, NULL, 0));
+  if (movie == NULL)
+    return;
   SWFDEC_SANDBOX (context->global)->MovieClip = movie;
   proto = swfdec_as_object_new (context);
+  if (!proto)
+    return;
   SWFDEC_AS_VALUE_SET_OBJECT (&val, proto);
   swfdec_as_object_set_variable_and_flags (movie,
       SWFDEC_AS_STR_prototype, &val, SWFDEC_AS_VARIABLE_HIDDEN |
