@@ -374,28 +374,6 @@ tag_func_sound_stream_head (SwfdecSwfDecoder *s, guint tag)
   return tag_func_enqueue (s, tag);
 }
 
-static int
-tag_func_metadata (SwfdecSwfDecoder *s, guint tag)
-{
-  char *meta;
-
-  meta = swfdec_bits_get_string (&s->b, s->version);
-  if (meta == NULL) {
-    SWFDEC_ERROR ("invalid metadata contents");
-    return SWFDEC_STATUS_OK;
-  }
-  SWFDEC_LOG ("metadata: %s", meta);
-
-  if (s->metadata) {
-    SWFDEC_ERROR ("Duplicated metadata tag, ignoring");
-    /* FIXME: or use this one? */
-    g_free (meta);
-  } else {
-    s->metadata = meta;
-  }
-  return SWFDEC_STATUS_OK;
-}
-
 struct tag_func_struct
 {
   const char *name;
@@ -442,7 +420,7 @@ static struct tag_func_struct tag_funcs[] = {
   [SWFDEC_TAG_DEFINEMOVIE] = {"DefineMovie", NULL, 0},
   [SWFDEC_TAG_DEFINESPRITE] = {"DefineSprite", tag_func_define_sprite, 0},
   [SWFDEC_TAG_NAMECHARACTER] = {"NameCharacter", NULL, 0},
-  [SWFDEC_TAG_PRODUCTINFO] = {"ProductInfo", NULL, 0},
+  [SWFDEC_TAG_SERIALNUMBER] = {"SerialNumber", NULL, 0},
   [SWFDEC_TAG_GENERATORTEXT] = {"GeneratorText", NULL, 0},
   [SWFDEC_TAG_FRAMELABEL] = {"FrameLabel", tag_func_frame_label, SWFDEC_TAG_DEFINE_SPRITE },
   [SWFDEC_TAG_SOUNDSTREAMHEAD2] = {"SoundStreamHead2", tag_func_sound_stream_head, SWFDEC_TAG_DEFINE_SPRITE },
@@ -469,10 +447,10 @@ static struct tag_func_struct tag_funcs[] = {
   [SWFDEC_TAG_DEFINEFONTALIGNZONES] = {"DefineFontAlignZones", NULL, 0},
   [SWFDEC_TAG_CSMTEXTSETTINGS] = {"CSMTextSettings", NULL, 0},
   [SWFDEC_TAG_DEFINEFONT3] = {"DefineFont3", tag_func_define_font_2, 0},
-  [SWFDEC_TAG_SYMBOLCLASS] = {"SymbolClass", NULL, SWFDEC_TAG_DEFINE_SPRITE },
-  [SWFDEC_TAG_METADATA] = {"Metadata", tag_func_metadata, 0},
+  [SWFDEC_TAG_AVM2DECL] = {"AVM2Decl", NULL, SWFDEC_TAG_DEFINE_SPRITE },
+  [SWFDEC_TAG_METADATA] = {"Metadata", NULL, 0},
   [SWFDEC_TAG_DEFINESCALINGGRID] = {"DefineScalingGrid", NULL, 0},
-  [SWFDEC_TAG_DOABC] = {"DoAbc", NULL, SWFDEC_TAG_DEFINE_SPRITE },
+  [SWFDEC_TAG_AVM2ACTION] = {"AVM2Action", NULL, SWFDEC_TAG_DEFINE_SPRITE },
   [SWFDEC_TAG_DEFINESHAPE4] = {"DefineShape4", tag_define_shape_4, 0},
   [SWFDEC_TAG_DEFINEMORPHSHAPE2] = {"DefineMorphShape2", NULL, 0},
   [SWFDEC_TAG_PRIVATE_IMAGE] = { "PrivateImage", NULL, 0},
