@@ -38,26 +38,27 @@ typedef struct {
   int			name_length;
   guint			index;
   guint			end_index;
-  SwfdecTextFormat	*format;
+  SwfdecTextFormat *	format;
 } ParserTag;
 
 typedef struct {
-  SwfdecAsContext	*cx;
+  SwfdecAsContext *	cx;
   gboolean		multiline;
   gboolean		condense_white;
-  SwfdecStyleSheet	*style_sheet;
-  GString *		text;
+  SwfdecStyleSheet *	style_sheet;
+  SwfdecTextBuffer *	text;
   GSList *		tags_open;
   GSList *		tags_closed;
 } ParserData;
 
 static void
-swfdec_text_field_movie_html_parse_close_tag (ParserData *data, ParserTag *tag)
+swfdec_text_field_movie_html_parse_close_tag (ParserData *data, ParserTag *tag,
+    gboolean end)
 {
   g_return_if_fail (data != NULL);
   g_return_if_fail (tag != NULL);
 
-  if (data->multiline &&
+  if (data->multiline && !end &&
       ((tag->name_length == 1 && !g_strncasecmp (tag->name, "p", 1)) ||
        (tag->name_length == 2 && !g_strncasecmp (tag->name, "li", 2))))
   {
@@ -71,7 +72,7 @@ swfdec_text_field_movie_html_parse_close_tag (ParserData *data, ParserTag *tag)
 	ParserTag *n = g_new0 (ParserTag, 1);
 	n->name = f->name;
 	n->name_length = f->name_length;
-	n->index = data->text->len;
+	n->index = swfdec_text_buffer_get_length (data->text);
 	n->end_index = n->index + 1;
 	if (f->format != NULL) {
 	  n->format = swfdec_text_format_copy (f->format);
@@ -82,10 +83,10 @@ swfdec_text_field_movie_html_parse_close_tag (ParserData *data, ParserTag *tag)
 	break;
       }
     }
-    data->text = g_string_append_c (data->text, '\n');
+    swfdec_text_buffer_append_text (data->text, "\n");
   }
 
-  tag->end_index = data->text->len;
+  tag->end_index = swfdec_text_buffer_get_length (data->text);
 
   data->tags_open = g_slist_remove (data->tags_open, tag);
   data->tags_closed = g_slist_prepend (data->tags_closed, tag);
@@ -312,11 +313,41 @@ swfdec_text_field_movie_html_parse_tag (ParserData *data, const char *p)
 
   if (close)
   {
-    if (data->tags_open != NULL) {
-      tag = data->tags_open->data;
-      if (name_length == tag->name_length &&
-	  !g_strncasecmp (name, tag->name, name_length))
-	swfdec_text_field_movie_html_parse_close_tag (data, tag);
+    if (name_length == 1 && !g_strncasecmp (name, "p", 1)) {
+      GSList *iter, *found;
+
+      found = NULL;
+      iter = data->tags_open;
+      while (iter != NULL) {
+	tag = iter->data;
+	if (tag->name_length == 1 && !g_strncasecmp (tag->name, "p", 1))
+	  found = iter;
+	iter = iter->next;
+      }
+
+      if (found != NULL) {
+	iter = data->tags_open;
+	while (iter != found) {
+	  tag = iter->data;
+	  iter = iter->next;
+	  if ((tag->name_length == 2 && !g_strncasecmp (tag->name, "li", 2)) ||
+	      (tag->name_length == 10 &&
+	       !g_strncasecmp (tag->name, "textformat", 10)))
+	    continue;
+	  swfdec_text_field_movie_html_parse_close_tag (data, tag, TRUE);
+	}
+	if (data->multiline)
+	  swfdec_text_buffer_append_text (data->text, "\n");
+	swfdec_text_field_movie_html_parse_close_tag (data, found->data,
+	    TRUE);
+      }
+    } else {
+      if (data->tags_open != NULL) {
+	tag = data->tags_open->data;
+	if (name_length == tag->name_length &&
+	    !g_strncasecmp (name, tag->name, name_length))
+	  swfdec_text_field_movie_html_parse_close_tag (data, tag, FALSE);
+      }
     }
 
     end = strchr (end, '>');
@@ -328,25 +359,20 @@ swfdec_text_field_movie_html_parse_tag (ParserData *data, const char *p)
     SwfdecAsObject *object;
     SwfdecAsValue val;
 
+    if (name_length == 3 && !g_strncasecmp (name, "tab", 3))
+      swfdec_text_buffer_append_text (data->text, "\t");
+
     if (data->multiline) {
       if (name_length == 2 && !g_strncasecmp (name, "br", 2))
       {
-	data->text = g_string_append_c (data->text, '\n');
+	swfdec_text_buffer_append_text (data->text, "\n");
       }
-      else if ((name_length == 1 && !g_strncasecmp (name, "p", 1)) ||
-	  (name_length == 2 && !g_strncasecmp (name, "li", 2)) ||
-	  (name_length == 2 && !g_strncasecmp (name, "br", 2)))
+      else if (name_length == 2 && !g_strncasecmp (name, "li", 1))
       {
-	    GSList *iter;
-
-	for (iter = data->tags_open; iter != NULL; iter = iter->next) {
-	  ParserTag *f = iter->data;
-	  if ((f->name_length == 1 && !g_strncasecmp (f->name, "p", 1)) ||
-	      (f->name_length == 2 && !g_strncasecmp (f->name, "li", 2))) {
-	    data->text = g_string_append_c (data->text, '\n');
-	    break;
-	  }
-	}
+	gsize length = swfdec_text_buffer_get_length (data->text);
+	const char *s = swfdec_text_buffer_get_text (data->text);
+	if (length > 0 && s[length-1] != '\n' && s[length-1] != '\r')
+	  swfdec_text_buffer_append_text (data->text, "\n");
       }
     }
 
@@ -354,7 +380,7 @@ swfdec_text_field_movie_html_parse_tag (ParserData *data, const char *p)
     tag->name = name;
     tag->name_length = name_length;
     tag->format = SWFDEC_TEXT_FORMAT (swfdec_text_format_new (data->cx));
-    tag->index = data->text->len;
+    tag->index = swfdec_text_buffer_get_length (data->text);
 
     data->tags_open = g_slist_prepend (data->tags_open, tag);
 
@@ -417,6 +443,22 @@ swfdec_text_field_movie_html_parse_text (ParserData *data, const char *p)
   g_return_val_if_fail (p != NULL, NULL);
   g_return_val_if_fail (*p != '\0' && *p != '<', NULL);
 
+  // condense the whitespace with previous text
+  // also skip any whitespace that would be preceding first actual text
+  if (data->condense_white && data->cx->version >= 8) {
+    gsize length = swfdec_text_buffer_get_length (data->text);
+    const char *s = swfdec_text_buffer_get_text (data->text);
+    if (length == 0 || g_ascii_isspace (s[length - 1]))
+      p += strspn (p, " \n\r\t");
+  }
+
+  // if it's only whitespace, don't add it
+  if (data->cx->version < 8) {
+    end = p + strspn (p, " \n\r\t");
+    if (*end == '\0' || *end == '<')
+      return end;
+  }
+
   // get the text
   // if condense_white: all whitespace blocks are converted to a single space
   while (*p != '\0' && *p != '<') {
@@ -429,11 +471,11 @@ swfdec_text_field_movie_html_parse_text (ParserData *data, const char *p)
     }
 
     unescaped = swfdec_xml_unescape_len (data->cx, p, end - p, TRUE);
-    data->text = g_string_append (data->text, unescaped);
+    swfdec_text_buffer_append_text (data->text, unescaped);
     g_free (unescaped);
 
     if (data->condense_white && g_ascii_isspace (*end)) {
-      data->text = g_string_append_c (data->text, ' ');
+      swfdec_text_buffer_append_text (data->text, " ");
       p = end + strspn (end, " \n\r\t");
     } else {
       p = end;
@@ -452,17 +494,15 @@ swfdec_text_field_movie_html_parse (SwfdecTextFieldMovie *text, const char *str)
   g_return_if_fail (SWFDEC_IS_TEXT_FIELD_MOVIE (text));
   g_return_if_fail (str != NULL);
 
-  text->input = g_string_assign (text->input, "");
-
   data.cx = SWFDEC_AS_OBJECT (text)->context;
-  data.multiline = (data.cx->version < 7 || text->text->multiline);
+  data.multiline = (data.cx->version < 7 || text->multiline);
   data.condense_white = text->condense_white;
   if (text->style_sheet != NULL && SWFDEC_IS_STYLESHEET (text->style_sheet)) {
     data.style_sheet = SWFDEC_STYLESHEET (text->style_sheet);
   } else {
     data.style_sheet = NULL;
   }
-  data.text = text->input;
+  data.text = text->text;
   data.tags_open = NULL;
   data.tags_closed = NULL;
 
@@ -481,8 +521,12 @@ swfdec_text_field_movie_html_parse (SwfdecTextFieldMovie *text, const char *str)
 
   // close remaining tags
   while (data.tags_open != NULL) {
+    /* yes, this really appends to the default format */
+    swfdec_text_attributes_copy (&text->default_attributes,
+	&((ParserTag *)data.tags_open->data)->format->attr,
+	((ParserTag *)data.tags_open->data)->format->values_set);
     swfdec_text_field_movie_html_parse_close_tag (&data,
-	(ParserTag *)data.tags_open->data);
+	(ParserTag *)data.tags_open->data, TRUE);
   }
 
   // add parsed styles
@@ -490,8 +534,8 @@ swfdec_text_field_movie_html_parse (SwfdecTextFieldMovie *text, const char *str)
     ParserTag *tag = (ParserTag *)data.tags_closed->data;
 
     if (tag->index != tag->end_index && tag->format != NULL) {
-      swfdec_text_field_movie_set_text_format (text, tag->format, tag->index,
-	  tag->end_index);
+      swfdec_text_buffer_set_attributes (text->text, tag->index,
+	  tag->end_index - tag->index, &tag->format->attr, tag->format->values_set);
     }
 
     g_free (tag);
@@ -539,8 +583,9 @@ static GString *
 swfdec_text_field_movie_html_text_append_paragraph (SwfdecTextFieldMovie *text,
     GString *string, guint start_index, guint end_index)
 {
-  SwfdecTextFormat *format, *format_prev, *format_font;
-  GSList *iter, *fonts, *iter_font;
+  const SwfdecTextAttributes *attr, *attr_prev, *attr_font;
+  SwfdecTextBufferIter *iter;
+  GSList *fonts, *iter_font;
   guint index_, index_prev;
   gboolean textformat, bullet, font = FALSE;
   char *escaped;
@@ -549,42 +594,39 @@ swfdec_text_field_movie_html_text_append_paragraph (SwfdecTextFieldMovie *text,
   g_return_val_if_fail (string != NULL, string);
   g_return_val_if_fail (start_index <= end_index, string);
 
-  g_return_val_if_fail (text->formats != NULL, string);
-  for (iter = text->formats; iter->next != NULL &&
-      ((SwfdecFormatIndex *)(iter->next->data))->index_ <= start_index;
-      iter = iter->next);
-
+  iter = swfdec_text_buffer_get_iter (text->text, start_index);
   index_ = start_index;
-  format = ((SwfdecFormatIndex *)(iter->data))->format;
+  attr = swfdec_text_buffer_iter_get_attributes (text->text, iter);
 
-  if (format->left_margin != 0 || format->right_margin != 0 ||
-      format->indent != 0 || format->leading != 0 ||
-      format->block_indent != 0 ||
-      swfdec_as_array_get_length (format->tab_stops) > 0)
+  if (attr->left_margin != 0 || attr->right_margin != 0 ||
+      attr->indent != 0 || attr->leading != 0 ||
+      attr->block_indent != 0 ||
+      attr->n_tab_stops > 0)
   {
     string = g_string_append (string, "<TEXTFORMAT");
-    if (format->left_margin) {
+    if (attr->left_margin) {
       g_string_append_printf (string, " LEFTMARGIN=\"%i\"",
-	  format->left_margin);
+	  attr->left_margin);
     }
-    if (format->right_margin) {
+    if (attr->right_margin) {
       g_string_append_printf (string, " RIGHTMARGIN=\"%i\"",
-	  format->right_margin);
+	  attr->right_margin);
     }
-    if (format->indent)
-      g_string_append_printf (string, " INDENT=\"%i\"", format->indent);
-    if (format->leading)
-      g_string_append_printf (string, " LEADING=\"%i\"", format->leading);
-    if (format->block_indent) {
+    if (attr->indent)
+      g_string_append_printf (string, " INDENT=\"%i\"", attr->indent);
+    if (attr->leading)
+      g_string_append_printf (string, " LEADING=\"%i\"", attr->leading);
+    if (attr->block_indent) {
       g_string_append_printf (string, " BLOCKINDENT=\"%i\"",
-	  format->block_indent);
+	  attr->block_indent);
     }
-    if (swfdec_as_array_get_length (format->tab_stops) > 0) {
-      SwfdecAsValue val;
-      SWFDEC_AS_VALUE_SET_OBJECT (&val, SWFDEC_AS_OBJECT  (format->tab_stops));
-      g_string_append_printf (string, " TABSTOPS=\"%s\"",
-	  swfdec_as_value_to_string (SWFDEC_AS_OBJECT
-	    (format->tab_stops)->context, &val));
+    if (attr->n_tab_stops > 0) {
+      guint i;
+      g_string_append (string, " TABSTOPS=\"\"");
+      for (i = 0; i < attr->n_tab_stops; i++) {
+	g_string_append_printf (string, "%d,", attr->tab_stops[i]);
+      }
+      string->str[string->len - 1] = '\"';
     }
     string = g_string_append (string, ">");
 
@@ -595,81 +637,81 @@ swfdec_text_field_movie_html_text_append_paragraph (SwfdecTextFieldMovie *text,
     textformat = FALSE;
   }
 
-  if (format->bullet) {
+  if (attr->bullet) {
     string = g_string_append (string, "<LI>");
     bullet = TRUE;
   } else {
     g_string_append_printf (string, "<P ALIGN=\"%s\">",
-	swfdec_text_field_movie_html_text_align_to_string (format->align));
+	swfdec_text_field_movie_html_text_align_to_string (attr->align));
     bullet = FALSE;
   }
 
-  // note we don't escape format->font, even thought it can have evil chars
+  // note we don't escape attr->font, even thought it can have evil chars
   g_string_append_printf (string, "<FONT FACE=\"%s\" SIZE=\"%i\" COLOR=\"#%06X\" LETTERSPACING=\"%i\" KERNING=\"%i\">",
-      format->font, format->size, format->color, (int)format->letter_spacing,
-      (format->kerning ? 1 : 0));
-  fonts = g_slist_prepend (NULL, format);
+      attr->font, attr->size, attr->color, (int)attr->letter_spacing,
+      (attr->kerning ? 1 : 0));
+  fonts = g_slist_prepend (NULL, (gpointer) attr);
 
-  if (format->url != SWFDEC_AS_STR_EMPTY)
+  if (attr->url != SWFDEC_AS_STR_EMPTY)
     g_string_append_printf (string, "<A HREF=\"%s\" TARGET=\"%s\">",
-	format->url, format->target);
-  if (format->bold)
+	attr->url, attr->target);
+  if (attr->bold)
     string = g_string_append (string, "<B>");
-  if (format->italic)
+  if (attr->italic)
     string = g_string_append (string, "<I>");
-  if (format->underline)
+  if (attr->underline)
     string = g_string_append (string, "<U>");
 
   // special case: use <= instead of < to add some extra markup
-  for (iter = iter->next;
-      iter != NULL && ((SwfdecFormatIndex *)(iter->data))->index_ <= end_index;
-      iter = iter->next)
+  for (iter = swfdec_text_buffer_iter_next (text->text, iter);
+      iter != NULL && swfdec_text_buffer_iter_get_start (text->text, iter) <= end_index;
+      iter = swfdec_text_buffer_iter_next (text->text, iter))
   {
     index_prev = index_;
-    format_prev = format;
-    index_ = ((SwfdecFormatIndex *)(iter->data))->index_;
-    format = ((SwfdecFormatIndex *)(iter->data))->format;
+    attr_prev = attr;
+    index_ = swfdec_text_buffer_iter_get_start (text->text, iter);
+    attr = swfdec_text_buffer_iter_get_attributes (text->text, iter);
 
-    escaped = swfdec_xml_escape_len (text->input->str + index_prev,
+    escaped = swfdec_xml_escape_len (swfdec_text_buffer_get_text (text->text) + index_prev,
 	index_ - index_prev);
     string = g_string_append (string, escaped);
     g_free (escaped);
     escaped = NULL;
 
     // Figure out what tags need to be rewritten
-    if (format->font != format_prev->font ||
-	format->size != format_prev->size ||
-	format->color != format_prev->color ||
-	(int)format->letter_spacing != (int)format_prev->letter_spacing ||
-	format->kerning != format_prev->kerning) {
+    if (attr->font != attr_prev->font ||
+	attr->size != attr_prev->size ||
+	attr->color != attr_prev->color ||
+	(int)attr->letter_spacing != (int)attr_prev->letter_spacing ||
+	attr->kerning != attr_prev->kerning) {
       font = TRUE;
-    } else if (format->url == format_prev->url &&
-	format->target == format_prev->target &&
-	format->bold == format_prev->bold &&
-	format->italic == format_prev->italic &&
-	format->underline == format_prev->underline) {
+    } else if (attr->url == attr_prev->url &&
+	attr->target == attr_prev->target &&
+	attr->bold == attr_prev->bold &&
+	attr->italic == attr_prev->italic &&
+	attr->underline == attr_prev->underline) {
       continue;
     }
 
     // Close tags
     for (iter_font = fonts; iter_font != NULL; iter_font = iter_font->next)
     {
-      format_font = (SwfdecTextFormat *)iter_font->data;
-      if (format->font == format_font->font &&
-	format->size == format_font->size &&
-	format->color == format_font->color &&
-	(int)format->letter_spacing == (int)format_font->letter_spacing &&
-	format->kerning == format_font->kerning) {
+      attr_font = iter_font->data;
+      if (attr->font == attr_font->font &&
+	attr->size == attr_font->size &&
+	attr->color == attr_font->color &&
+	(int)attr->letter_spacing == (int)attr_font->letter_spacing &&
+	attr->kerning == attr_font->kerning) {
 	break;
       }
     }
-    if (format_prev->underline)
+    if (attr_prev->underline)
       string = g_string_append (string, "</U>");
-    if (format_prev->italic)
+    if (attr_prev->italic)
       string = g_string_append (string, "</I>");
-    if (format_prev->bold)
+    if (attr_prev->bold)
       string = g_string_append (string, "</B>");
-    if (format_prev->url != SWFDEC_AS_STR_EMPTY)
+    if (attr_prev->url != SWFDEC_AS_STR_EMPTY)
       string = g_string_append (string, "</A>");
     if (iter_font != NULL) {
       while (fonts != iter_font) {
@@ -679,59 +721,59 @@ swfdec_text_field_movie_html_text_append_paragraph (SwfdecTextFieldMovie *text,
     }
 
     // Open tags
-    format_font = (SwfdecTextFormat *)fonts->data;
-    if (font && (format->font != format_font->font ||
-	 format->size != format_font->size ||
-	 format->color != format_font->color ||
-	 (int)format->letter_spacing != (int)format_font->letter_spacing ||
-	 format->kerning != format_font->kerning))
+    attr_font = fonts->data;
+    if (font && (attr->font != attr_font->font ||
+	 attr->size != attr_font->size ||
+	 attr->color != attr_font->color ||
+	 (int)attr->letter_spacing != (int)attr_font->letter_spacing ||
+	 attr->kerning != attr_font->kerning))
     {
-      fonts = g_slist_prepend (fonts, format);
+      fonts = g_slist_prepend (fonts, (gpointer) attr);
 
       string = g_string_append (string, "<FONT");
-      // note we don't escape format->font, even thought it can have evil chars
-      if (format->font != format_font->font)
-	g_string_append_printf (string, " FACE=\"%s\"", format->font);
-      if (format->size != format_font->size)
-	g_string_append_printf (string, " SIZE=\"%i\"", format->size);
-      if (format->color != format_font->color)
-	g_string_append_printf (string, " COLOR=\"#%06X\"", format->color);
-      if ((int)format->letter_spacing != (int)format_font->letter_spacing) {
+      // note we don't escape attr->font, even thought it can have evil chars
+      if (attr->font != attr_font->font)
+	g_string_append_printf (string, " FACE=\"%s\"", attr->font);
+      if (attr->size != attr_font->size)
+	g_string_append_printf (string, " SIZE=\"%i\"", attr->size);
+      if (attr->color != attr_font->color)
+	g_string_append_printf (string, " COLOR=\"#%06X\"", attr->color);
+      if ((int)attr->letter_spacing != (int)attr_font->letter_spacing) {
 	g_string_append_printf (string, " LETTERSPACING=\"%i\"",
-	    (int)format->letter_spacing);
+	    (int)attr->letter_spacing);
       }
-      if (format->kerning != format_font->kerning) {
+      if (attr->kerning != attr_font->kerning) {
 	g_string_append_printf (string, " KERNING=\"%i\"",
-	    (format->kerning ? 1 : 0));
+	    (attr->kerning ? 1 : 0));
       }
       string = g_string_append (string, ">");
     }
-    if (format->url != SWFDEC_AS_STR_EMPTY) {
+    if (attr->url != SWFDEC_AS_STR_EMPTY) {
       g_string_append_printf (string, "<A HREF=\"%s\" TARGET=\"%s\">",
-	  format->url, format->target);
+	  attr->url, attr->target);
     }
-    if (format->bold)
+    if (attr->bold)
       string = g_string_append (string, "<B>");
-    if (format->italic)
+    if (attr->italic)
       string = g_string_append (string, "<I>");
-    if (format->underline)
+    if (attr->underline)
       string = g_string_append (string, "<U>");
   }
 
-  escaped = swfdec_xml_escape_len (text->input->str + index_,
+  escaped = swfdec_xml_escape_len (swfdec_text_buffer_get_text (text->text) + index_,
       end_index - index_);
   string = g_string_append (string, escaped);
   g_free (escaped);
 
-  if (format->underline)
+  if (attr->underline)
     string = g_string_append (string, "</U>");
-  if (format->italic)
+  if (attr->italic)
     string = g_string_append (string, "</I>");
-  if (format->bold)
+  if (attr->bold)
     string = g_string_append (string, "</B>");
-  if (format->url != SWFDEC_AS_STR_EMPTY)
+  if (attr->url != SWFDEC_AS_STR_EMPTY)
     string = g_string_append (string, "</A>");
-  for (iter = fonts; iter != NULL; iter = iter->next)
+  for (iter_font = fonts; iter_font != NULL; iter_font = iter_font->next)
     string = g_string_append (string, "</FONT>");
   g_slist_free (fonts);
   if (bullet) {
@@ -748,25 +790,22 @@ swfdec_text_field_movie_html_text_append_paragraph (SwfdecTextFieldMovie *text,
 const char *
 swfdec_text_field_movie_get_html_text (SwfdecTextFieldMovie *text)
 {
-  const char *p, *end;
+  const char *p, *end, *start;
   GString *string;
 
   g_return_val_if_fail (SWFDEC_IS_TEXT_FIELD_MOVIE (text),
       SWFDEC_AS_STR_EMPTY);
 
-  if (text->input == NULL)
-    return SWFDEC_AS_STR_EMPTY;
-
   string = g_string_new ("");
+  p = start = swfdec_text_buffer_get_text (text->text);
 
-  p = text->input->str;
   while (*p != '\0') {
     end = strpbrk (p, "\r\n");
     if (end == NULL)
       end = strchr (p, '\0');
 
     string = swfdec_text_field_movie_html_text_append_paragraph (text, string,
-	p - text->input->str, end - text->input->str);
+	p - start, end - start);
 
     p = end;
     if (*p != '\0') p++;
