@@ -64,21 +64,21 @@ static int property_offsets[] = {
 };
 
 static void
-swfdec_text_format_mark (SwfdecGcObject *object)
+swfdec_text_format_do_mark (SwfdecAsObject *object)
 {
   SwfdecTextFormat *format = SWFDEC_TEXT_FORMAT (object);
 
   swfdec_text_attributes_mark (&format->attr);
 
-  SWFDEC_GC_OBJECT_CLASS (swfdec_text_format_parent_class)->mark (object);
+  SWFDEC_AS_OBJECT_CLASS (swfdec_text_format_parent_class)->mark (object);
 }
 
 static void
 swfdec_text_format_class_init (SwfdecTextFormatClass *klass)
 {
-  SwfdecGcObjectClass *gc_class = SWFDEC_GC_OBJECT_CLASS (klass);
+  SwfdecAsObjectClass *asobject_class = SWFDEC_AS_OBJECT_CLASS (klass);
 
-  gc_class->mark = swfdec_text_format_mark;
+  asobject_class->mark = swfdec_text_format_do_mark;
 }
 
 static void
@@ -111,7 +111,6 @@ swfdec_text_format_set_string (SwfdecAsObject *object,
     SwfdecTextAttribute property, guint argc, SwfdecAsValue *argv)
 {
   SwfdecTextFormat *format;
-  SwfdecAsContext *context;
   const char *s;
 
   if (!SWFDEC_IS_TEXT_FORMAT (object))
@@ -121,10 +120,9 @@ swfdec_text_format_set_string (SwfdecAsObject *object,
   if (argc < 1)
     return;
 
-  context = swfdec_gc_object_get_context (format);
-  swfdec_as_value_to_integer (context, &argv[0]);
-  swfdec_as_value_to_number (context, &argv[0]);
-  s = swfdec_as_value_to_string (context, &argv[0]);
+  swfdec_as_value_to_integer (object->context, &argv[0]);
+  swfdec_as_value_to_number (object->context, &argv[0]);
+  s = swfdec_as_value_to_string (object->context, &argv[0]);
 
   if (SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0]) ||
       SWFDEC_AS_VALUE_IS_NULL (&argv[0])) {
@@ -164,7 +162,6 @@ swfdec_text_format_set_boolean (SwfdecAsObject *object,
     SwfdecTextAttribute property, guint argc, SwfdecAsValue *argv)
 {
   SwfdecTextFormat *format;
-  SwfdecAsContext *context;
 
   if (!SWFDEC_IS_TEXT_FORMAT (object))
     return;
@@ -173,17 +170,16 @@ swfdec_text_format_set_boolean (SwfdecAsObject *object,
   if (argc < 1)
     return;
 
-  context = swfdec_gc_object_get_context (format);
-  swfdec_as_value_to_integer (context, &argv[0]);
-  swfdec_as_value_to_number (context, &argv[0]);
-  swfdec_as_value_to_string (context, &argv[0]);
+  swfdec_as_value_to_integer (object->context, &argv[0]);
+  swfdec_as_value_to_number (object->context, &argv[0]);
+  swfdec_as_value_to_string (object->context, &argv[0]);
 
   if (SWFDEC_AS_VALUE_IS_UNDEFINED (&argv[0]) ||
       SWFDEC_AS_VALUE_IS_NULL (&argv[0])) {
     SWFDEC_TEXT_ATTRIBUTE_UNSET (format->values_set, property);
   } else {
     G_STRUCT_MEMBER (gboolean, format, property_offsets[property]) =
-      swfdec_as_value_to_boolean (context, &argv[0]);
+      swfdec_as_value_to_boolean (object->context, &argv[0]);
     SWFDEC_TEXT_ATTRIBUTE_SET (format->values_set, property);
   }
 }
@@ -266,8 +262,8 @@ swfdec_text_format_set_integer (SwfdecAsObject *object,
     SWFDEC_TEXT_ATTRIBUTE_UNSET (format->values_set, property);
   } else {
     G_STRUCT_MEMBER (int, format, property_offsets[property]) =
-      swfdec_text_format_value_to_integer (swfdec_gc_object_get_context (format),
-	  &argv[0], allow_negative);
+      swfdec_text_format_value_to_integer (object->context, &argv[0],
+	  allow_negative);
     SWFDEC_TEXT_ATTRIBUTE_SET (format->values_set, property);
   }
 }
@@ -821,9 +817,11 @@ swfdec_text_format_getTextExtent (SwfdecAsContext *cx, SwfdecAsObject *object,
   SWFDEC_AS_CHECK (SWFDEC_TYPE_TEXT_FORMAT, &format, "s", &text);
 
   obj = swfdec_as_object_new_empty (cx);
+  if (obj == NULL)
+    return;
 
   buffer = swfdec_text_buffer_new ();
-  swfdec_text_buffer_set_default_attributes (buffer,
+  swfdec_text_buffer_set_attributes (buffer, 0, 0, 
       &format->attr, format->values_set);
   swfdec_text_buffer_append_text (buffer, text);
   layout = swfdec_text_layout_new (buffer);
@@ -908,7 +906,7 @@ swfdec_text_format_set_defaults (SwfdecTextFormat *format)
   swfdec_text_attributes_reset (&format->attr);
   format->values_set = SWFDEC_TEXT_ATTRIBUTES_MASK;
 
-  if (swfdec_gc_object_get_context (format)->version < 8) {
+  if (SWFDEC_AS_OBJECT (format)->context->version < 8) {
     SWFDEC_TEXT_ATTRIBUTE_UNSET (format->values_set, SWFDEC_TEXT_ATTRIBUTE_KERNING);
     SWFDEC_TEXT_ATTRIBUTE_UNSET (format->values_set, SWFDEC_TEXT_ATTRIBUTE_LETTER_SPACING);
   }
@@ -1048,7 +1046,7 @@ swfdec_text_format_construct (SwfdecAsContext *cx, SwfdecAsObject *object,
 }
 
 SwfdecTextFormat *
-swfdec_text_format_copy (SwfdecTextFormat *copy_from)
+swfdec_text_format_copy (const SwfdecTextFormat *copy_from)
 {
   SwfdecAsObject *object_to;
   SwfdecTextFormat *copy_to;
@@ -1056,7 +1054,7 @@ swfdec_text_format_copy (SwfdecTextFormat *copy_from)
   g_return_val_if_fail (SWFDEC_IS_TEXT_FORMAT (copy_from), NULL);
 
   object_to = swfdec_text_format_new_no_properties (
-      swfdec_gc_object_get_context (copy_from));
+      SWFDEC_AS_OBJECT (copy_from)->context);
   if (object_to == NULL)
     return NULL;
   copy_to = SWFDEC_TEXT_FORMAT (object_to);
@@ -1076,7 +1074,11 @@ swfdec_text_format_new_no_properties (SwfdecAsContext *context)
 
   g_return_val_if_fail (SWFDEC_IS_AS_CONTEXT (context), NULL);
 
-  ret = g_object_new (SWFDEC_TYPE_TEXT_FORMAT, "context", context, NULL);
+  if (!swfdec_as_context_use_mem (context, sizeof (SwfdecTextFormat)))
+    return NULL;
+
+  ret = g_object_new (SWFDEC_TYPE_TEXT_FORMAT, NULL);
+  swfdec_as_object_add (ret, context, sizeof (SwfdecTextFormat));
 
   swfdec_text_format_clear (SWFDEC_TEXT_FORMAT (ret));
 
