@@ -5,8 +5,6 @@
 #include <glib-object.h>
 #include <cairo.h>
 
-G_BEGIN_DECLS
-
 /* Pixel value in the same colorspace as cairo - endian-dependant ARGB.
  * The alpha pixel must be present */
 typedef guint SwfdecColor;
@@ -33,9 +31,8 @@ typedef int SwfdecFixed;
 #define SWFDEC_FIXED_TO_INT(f) ((f) / SWFDEC_FIXED_SCALE_FACTOR)
 #define SWFDEC_INT_TO_FIXED(i) ((i) * SWFDEC_FIXED_SCALE_FACTOR)
 
+typedef struct _SwfdecActor SwfdecActor;
 typedef struct _SwfdecButton SwfdecButton;
-typedef struct _SwfdecCache SwfdecCache;
-typedef struct _SwfdecCacheHandle SwfdecCacheHandle;
 typedef struct _SwfdecCharacter SwfdecCharacter;
 typedef struct _SwfdecColorTransform SwfdecColorTransform;
 typedef struct _SwfdecDecoder SwfdecDecoder;
@@ -53,6 +50,7 @@ typedef struct _SwfdecShapeVec SwfdecShapeVec;
 typedef struct _SwfdecRect SwfdecRect;
 typedef struct _SwfdecResource SwfdecResource;
 typedef struct _SwfdecRootSprite SwfdecRootSprite;
+typedef struct _SwfdecSandbox SwfdecSandbox;
 typedef struct _SwfdecScriptable SwfdecScriptable;
 typedef struct _SwfdecSound SwfdecSound;
 typedef struct _SwfdecSoundChunk SwfdecSoundChunk;
@@ -62,5 +60,4 @@ typedef struct _SwfdecSpriteMovie SwfdecSpriteMovie;
 typedef struct _SwfdecSwfDecoder SwfdecSwfDecoder;
 typedef struct _SwfdecText SwfdecText;
 
-G_END_DECLS
 #endif
