@@ -73,8 +73,12 @@ swfdec_video_movie_init_context (SwfdecPlayer *player)
   context = SWFDEC_AS_CONTEXT (player);
   video = SWFDEC_AS_OBJECT (swfdec_as_object_add_function (context->global, 
       SWFDEC_AS_STR_Video, 0, NULL, 0));
+  if (video == NULL)
+    return;
   SWFDEC_SANDBOX (context->global)->Video = video;
   proto = swfdec_as_object_new_empty (context);
+  if (proto == NULL)
+    return;
   /* set the right properties on the Video object */
   SWFDEC_AS_VALUE_SET_OBJECT (&val, proto);
   swfdec_as_object_set_variable_and_flags (video, SWFDEC_AS_STR_prototype, &val,
